"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import date, datetime
from kintsugi_tax_platform_sdk import errors, models, utils
from kintsugi_tax_platform_sdk._hooks import HookContext
from kintsugi_tax_platform_sdk.types import Nullable, OptionalNullable, UNSET
from kintsugi_tax_platform_sdk.utils.unmarshal_json_response import (
    unmarshal_json_response,
)
from typing import Any, List, Mapping, Optional, Union


class Customers(BaseSDK):
    def list(
        self,
        *,
        security: Union[
            models.GetCustomersV1Security, models.GetCustomersV1SecurityTypedDict
        ],
        x_organization_id: Nullable[str],
        search_query: OptionalNullable[str] = UNSET,
        country: OptionalNullable[
            Union[
                List[models.GetCustomersV1Country],
                List[models.GetCustomersV1CountryTypedDict],
            ]
        ] = UNSET,
        state: OptionalNullable[str] = UNSET,
        source_in: OptionalNullable[str] = UNSET,
        order_by: OptionalNullable[str] = UNSET,
        page: Optional[int] = 1,
        size: Optional[int] = 50,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PageCustomerRead:
        r"""Get Customers

        The Get Customers API retrieves
        a paginated list of customers based on specified filters.
        This API allows searching, filtering by country and state, and sorting the results.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param search_query: Search term to filter customers by name or other details
        :param country: Country code in ISO 3166-1 alpha-2 format (e.g., 'US')
        :param state: State or province code to filter customers
        :param source_in: Filter customers by source (comma-separated)
        :param order_by: Comma-separated list of fields to sort results by.
        :param page: Page number
        :param size: Page size
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCustomersV1Request(
            search_query=search_query,
            country=country,
            state=state,
            source_in=source_in,
            order_by=order_by,
            page=page,
            size=size,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/customers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(security, models.GetCustomersV1Security),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_customers_v1",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.PageCustomerRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcCustomersResponsesValidationErrorResponseData, http_res
            )
            raise errors.BackendSrcCustomersResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        security: Union[
            models.GetCustomersV1Security, models.GetCustomersV1SecurityTypedDict
        ],
        x_organization_id: Nullable[str],
        search_query: OptionalNullable[str] = UNSET,
        country: OptionalNullable[
            Union[
                List[models.GetCustomersV1Country],
                List[models.GetCustomersV1CountryTypedDict],
            ]
        ] = UNSET,
        state: OptionalNullable[str] = UNSET,
        source_in: OptionalNullable[str] = UNSET,
        order_by: OptionalNullable[str] = UNSET,
        page: Optional[int] = 1,
        size: Optional[int] = 50,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PageCustomerRead:
        r"""Get Customers

        The Get Customers API retrieves
        a paginated list of customers based on specified filters.
        This API allows searching, filtering by country and state, and sorting the results.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param search_query: Search term to filter customers by name or other details
        :param country: Country code in ISO 3166-1 alpha-2 format (e.g., 'US')
        :param state: State or province code to filter customers
        :param source_in: Filter customers by source (comma-separated)
        :param order_by: Comma-separated list of fields to sort results by.
        :param page: Page number
        :param size: Page size
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCustomersV1Request(
            search_query=search_query,
            country=country,
            state=state,
            source_in=source_in,
            order_by=order_by,
            page=page,
            size=size,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/customers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(security, models.GetCustomersV1Security),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_customers_v1",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.PageCustomerRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcCustomersResponsesValidationErrorResponseData, http_res
            )
            raise errors.BackendSrcCustomersResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        security: Union[
            models.CreateCustomerV1CustomersPostSecurity,
            models.CreateCustomerV1CustomersPostSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        phone: OptionalNullable[str] = UNSET,
        street_1: OptionalNullable[str] = UNSET,
        street_2: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        full_address: OptionalNullable[str] = UNSET,
        name: OptionalNullable[str] = UNSET,
        external_id: OptionalNullable[str] = UNSET,
        status: Optional[models.StatusEnum] = None,
        email: OptionalNullable[str] = UNSET,
        source: OptionalNullable[models.SourceEnum] = UNSET,
        connection_id: OptionalNullable[str] = UNSET,
        address_status: Optional[models.AddressStatus] = None,
        registration_number: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerRead:
        r"""Create Customer

        The Create Customer API enables the creation of a new customer record with essential
        details like name, contact information, and address, along with optional metadata.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param phone: Customer's phone number
        :param street_1: Primary street address.
        :param street_2: Additional street address details, such as an apartment or suite number.
        :param city: City where the customer resides.
        :param county: County or district of the customer.
        :param state: State or province of the customer.
        :param postal_code: ZIP or Postal code of the customer.
        :param country: Country code in ISO 3166-1 alpha-2 format
        :param full_address: Complete address string of the customer, which can be used as an alternative to individual fields.
        :param name: Name of the customer.
        :param external_id: External identifier associated with the customer.
        :param status:
        :param email: Customer's email address
        :param source: Source of the customer's record.
        :param connection_id: Identifier for the connection source, if applicable.
        :param address_status:
        :param registration_number: Registration number of the customer.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateCustomerV1CustomersPostRequest(
            x_organization_id=x_organization_id,
            customer_create=models.CustomerCreate(
                phone=phone,
                street_1=street_1,
                street_2=street_2,
                city=city,
                county=county,
                state=state,
                postal_code=postal_code,
                country=country,
                full_address=full_address,
                name=name,
                external_id=external_id,
                status=status,
                email=email,
                source=source,
                connection_id=connection_id,
                address_status=address_status,
                registration_number=registration_number,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/customers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CreateCustomerV1CustomersPostSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.customer_create, False, False, "json", models.CustomerCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_customer_v1_customers_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcCustomersResponsesValidationErrorResponseData, http_res
            )
            raise errors.BackendSrcCustomersResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        security: Union[
            models.CreateCustomerV1CustomersPostSecurity,
            models.CreateCustomerV1CustomersPostSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        phone: OptionalNullable[str] = UNSET,
        street_1: OptionalNullable[str] = UNSET,
        street_2: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        full_address: OptionalNullable[str] = UNSET,
        name: OptionalNullable[str] = UNSET,
        external_id: OptionalNullable[str] = UNSET,
        status: Optional[models.StatusEnum] = None,
        email: OptionalNullable[str] = UNSET,
        source: OptionalNullable[models.SourceEnum] = UNSET,
        connection_id: OptionalNullable[str] = UNSET,
        address_status: Optional[models.AddressStatus] = None,
        registration_number: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerRead:
        r"""Create Customer

        The Create Customer API enables the creation of a new customer record with essential
        details like name, contact information, and address, along with optional metadata.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param phone: Customer's phone number
        :param street_1: Primary street address.
        :param street_2: Additional street address details, such as an apartment or suite number.
        :param city: City where the customer resides.
        :param county: County or district of the customer.
        :param state: State or province of the customer.
        :param postal_code: ZIP or Postal code of the customer.
        :param country: Country code in ISO 3166-1 alpha-2 format
        :param full_address: Complete address string of the customer, which can be used as an alternative to individual fields.
        :param name: Name of the customer.
        :param external_id: External identifier associated with the customer.
        :param status:
        :param email: Customer's email address
        :param source: Source of the customer's record.
        :param connection_id: Identifier for the connection source, if applicable.
        :param address_status:
        :param registration_number: Registration number of the customer.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateCustomerV1CustomersPostRequest(
            x_organization_id=x_organization_id,
            customer_create=models.CustomerCreate(
                phone=phone,
                street_1=street_1,
                street_2=street_2,
                city=city,
                county=county,
                state=state,
                postal_code=postal_code,
                country=country,
                full_address=full_address,
                name=name,
                external_id=external_id,
                status=status,
                email=email,
                source=source,
                connection_id=connection_id,
                address_status=address_status,
                registration_number=registration_number,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/customers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CreateCustomerV1CustomersPostSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.customer_create, False, False, "json", models.CustomerCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_customer_v1_customers_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcCustomersResponsesValidationErrorResponseData, http_res
            )
            raise errors.BackendSrcCustomersResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        security: Union[
            models.GetCustomerByIDV1CustomersCustomerIDGetSecurity,
            models.GetCustomerByIDV1CustomersCustomerIDGetSecurityTypedDict,
        ],
        customer_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerRead:
        r"""Get Customer By Id

        The Get Customer By ID API retrieves the details of a single customer
        using their unique identifier. It returns customer-specific data,
        including contact information, address, name and metadata, etc.

        :param security:
        :param customer_id: Unique identifier of the customer
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCustomerByIDV1CustomersCustomerIDGetRequest(
            customer_id=customer_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/customers/{customer_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetCustomerByIDV1CustomersCustomerIDGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_customer_by_id_v1_customers__customer_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        security: Union[
            models.GetCustomerByIDV1CustomersCustomerIDGetSecurity,
            models.GetCustomerByIDV1CustomersCustomerIDGetSecurityTypedDict,
        ],
        customer_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerRead:
        r"""Get Customer By Id

        The Get Customer By ID API retrieves the details of a single customer
        using their unique identifier. It returns customer-specific data,
        including contact information, address, name and metadata, etc.

        :param security:
        :param customer_id: Unique identifier of the customer
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCustomerByIDV1CustomersCustomerIDGetRequest(
            customer_id=customer_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/customers/{customer_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetCustomerByIDV1CustomersCustomerIDGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_customer_by_id_v1_customers__customer_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        security: Union[
            models.UpdateCustomerV1CustomersCustomerIDPutSecurity,
            models.UpdateCustomerV1CustomersCustomerIDPutSecurityTypedDict,
        ],
        customer_id: str,
        x_organization_id: Nullable[str],
        phone: OptionalNullable[str] = UNSET,
        street_1: OptionalNullable[str] = UNSET,
        street_2: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        full_address: OptionalNullable[str] = UNSET,
        name: OptionalNullable[str] = UNSET,
        status: OptionalNullable[models.StatusEnum] = UNSET,
        email: OptionalNullable[str] = UNSET,
        source: OptionalNullable[models.SourceEnum] = UNSET,
        address_status: Optional[models.AddressStatus] = None,
        external_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerRead:
        r"""Update Customer

        The Update Customer API allows you to modify an existing customer's
        information using their unique identifier,
        enabling updates to their details as needed.

        :param security:
        :param customer_id: Unique identifier of the customer to be retrieved.
        :param x_organization_id: The unique identifier for the organization making the request
        :param phone: Phone number associated with the customer.
        :param street_1: Primary street address.
        :param street_2: Additional street address details, such as an apartment or suite number.
        :param city: City where the customer resides.
        :param county: County or district of the customer.
        :param state: State or province of the customer.
        :param postal_code: ZIP or Postal code of the customer.
        :param country: Country code in ISO 3166-1 alpha-2 format
        :param full_address: Complete address string of the customer, which can be used as an alternative to individual fields.
        :param name: Name of the customer.
        :param status: Status of the customer.
        :param email: Email address of the customer.
        :param source: Source of the customer's record
        :param address_status:
        :param external_id: External identifier associated with the customer
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateCustomerV1CustomersCustomerIDPutRequest(
            customer_id=customer_id,
            x_organization_id=x_organization_id,
            customer_update=models.CustomerUpdate(
                phone=phone,
                street_1=street_1,
                street_2=street_2,
                city=city,
                county=county,
                state=state,
                postal_code=postal_code,
                country=country,
                full_address=full_address,
                name=name,
                status=status,
                email=email,
                source=source,
                address_status=address_status,
                external_id=external_id,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/customers/{customer_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.UpdateCustomerV1CustomersCustomerIDPutSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.customer_update, False, False, "json", models.CustomerUpdate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_customer_v1_customers__customer_id__put",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcCustomersResponsesValidationErrorResponseData, http_res
            )
            raise errors.BackendSrcCustomersResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        security: Union[
            models.UpdateCustomerV1CustomersCustomerIDPutSecurity,
            models.UpdateCustomerV1CustomersCustomerIDPutSecurityTypedDict,
        ],
        customer_id: str,
        x_organization_id: Nullable[str],
        phone: OptionalNullable[str] = UNSET,
        street_1: OptionalNullable[str] = UNSET,
        street_2: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        full_address: OptionalNullable[str] = UNSET,
        name: OptionalNullable[str] = UNSET,
        status: OptionalNullable[models.StatusEnum] = UNSET,
        email: OptionalNullable[str] = UNSET,
        source: OptionalNullable[models.SourceEnum] = UNSET,
        address_status: Optional[models.AddressStatus] = None,
        external_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerRead:
        r"""Update Customer

        The Update Customer API allows you to modify an existing customer's
        information using their unique identifier,
        enabling updates to their details as needed.

        :param security:
        :param customer_id: Unique identifier of the customer to be retrieved.
        :param x_organization_id: The unique identifier for the organization making the request
        :param phone: Phone number associated with the customer.
        :param street_1: Primary street address.
        :param street_2: Additional street address details, such as an apartment or suite number.
        :param city: City where the customer resides.
        :param county: County or district of the customer.
        :param state: State or province of the customer.
        :param postal_code: ZIP or Postal code of the customer.
        :param country: Country code in ISO 3166-1 alpha-2 format
        :param full_address: Complete address string of the customer, which can be used as an alternative to individual fields.
        :param name: Name of the customer.
        :param status: Status of the customer.
        :param email: Email address of the customer.
        :param source: Source of the customer's record
        :param address_status:
        :param external_id: External identifier associated with the customer
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateCustomerV1CustomersCustomerIDPutRequest(
            customer_id=customer_id,
            x_organization_id=x_organization_id,
            customer_update=models.CustomerUpdate(
                phone=phone,
                street_1=street_1,
                street_2=street_2,
                city=city,
                county=county,
                state=state,
                postal_code=postal_code,
                country=country,
                full_address=full_address,
                name=name,
                status=status,
                email=email,
                source=source,
                address_status=address_status,
                external_id=external_id,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/customers/{customer_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.UpdateCustomerV1CustomersCustomerIDPutSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.customer_update, False, False, "json", models.CustomerUpdate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_customer_v1_customers__customer_id__put",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcCustomersResponsesValidationErrorResponseData, http_res
            )
            raise errors.BackendSrcCustomersResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_by_external_id(
        self,
        *,
        security: Union[
            models.GetCustomerByExternalIDV1CustomersExternalExternalIDGetSecurity,
            models.GetCustomerByExternalIDV1CustomersExternalExternalIDGetSecurityTypedDict,
        ],
        external_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerRead:
        r"""Get Customer By External Id

        The Get Customer By External ID API retrieves the details of a single customer using
        their external identifier. This endpoint is useful for accessing customer data when only
        an external ID is available.

        :param security:
        :param external_id: The external identifier of the customer to retrieve.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCustomerByExternalIDV1CustomersExternalExternalIDGetRequest(
            external_id=external_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/customers/external/{external_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetCustomerByExternalIDV1CustomersExternalExternalIDGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_customer_by_external_id_v1_customers_external__external_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_by_external_id_async(
        self,
        *,
        security: Union[
            models.GetCustomerByExternalIDV1CustomersExternalExternalIDGetSecurity,
            models.GetCustomerByExternalIDV1CustomersExternalExternalIDGetSecurityTypedDict,
        ],
        external_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerRead:
        r"""Get Customer By External Id

        The Get Customer By External ID API retrieves the details of a single customer using
        their external identifier. This endpoint is useful for accessing customer data when only
        an external ID is available.

        :param security:
        :param external_id: The external identifier of the customer to retrieve.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCustomerByExternalIDV1CustomersExternalExternalIDGetRequest(
            external_id=external_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/customers/external/{external_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetCustomerByExternalIDV1CustomersExternalExternalIDGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_customer_by_external_id_v1_customers_external__external_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_transactions(
        self,
        *,
        security: Union[
            models.GetTransactionsByCustomerIDV1CustomersCustomerIDTransactionsGetSecurity,
            models.GetTransactionsByCustomerIDV1CustomersCustomerIDTransactionsGetSecurityTypedDict,
        ],
        customer_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.TransactionRead]:
        r"""Get Transactions By Customer Id

        Get a list of transactions for a customer by their unique ID.

        :param security:
        :param customer_id:
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTransactionsByCustomerIDV1CustomersCustomerIDTransactionsGetRequest(
            customer_id=customer_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/customers/{customer_id}/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetTransactionsByCustomerIDV1CustomersCustomerIDTransactionsGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transactions_by_customer_id_v1_customers__customer_id__transactions_get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.TransactionRead], http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_transactions_async(
        self,
        *,
        security: Union[
            models.GetTransactionsByCustomerIDV1CustomersCustomerIDTransactionsGetSecurity,
            models.GetTransactionsByCustomerIDV1CustomersCustomerIDTransactionsGetSecurityTypedDict,
        ],
        customer_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.TransactionRead]:
        r"""Get Transactions By Customer Id

        Get a list of transactions for a customer by their unique ID.

        :param security:
        :param customer_id:
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTransactionsByCustomerIDV1CustomersCustomerIDTransactionsGetRequest(
            customer_id=customer_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/customers/{customer_id}/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetTransactionsByCustomerIDV1CustomersCustomerIDTransactionsGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transactions_by_customer_id_v1_customers__customer_id__transactions_get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.TransactionRead], http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def create_transaction(
        self,
        *,
        security: Union[
            models.CreateTransactionByCustomerIDV1CustomersCustomerIDTransactionsPostSecurity,
            models.CreateTransactionByCustomerIDV1CustomersCustomerIDTransactionsPostSecurityTypedDict,
        ],
        customer_id_param: str,
        x_organization_id: Nullable[str],
        organization_id: str,
        external_id: str,
        date_: datetime,
        addresses: Union[
            models.TransactionCreateAddresses,
            models.TransactionCreateAddressesTypedDict,
        ],
        transaction_items: Union[
            List[models.TransactionItemCreateUpdate],
            List[models.TransactionItemCreateUpdateTypedDict],
        ],
        requires_exemption: OptionalNullable[
            Union[models.ExemptionRequired, models.ExemptionRequiredTypedDict]
        ] = UNSET,
        shop_date: OptionalNullable[date] = UNSET,
        shop_date_tz: OptionalNullable[str] = UNSET,
        status: Optional[models.TransactionStatusEnum] = None,
        description: OptionalNullable[str] = UNSET,
        refund_status: OptionalNullable[models.TransactionRefundStatus] = UNSET,
        total_amount: Optional[
            Union[
                models.TransactionCreateTotalAmount,
                models.TransactionCreateTotalAmountTypedDict,
            ]
        ] = None,
        customer_id: OptionalNullable[str] = UNSET,
        marketplace: OptionalNullable[bool] = UNSET,
        exempt: OptionalNullable[models.TransactionExemptStatusEnum] = UNSET,
        exemptions: OptionalNullable[
            Union[List[models.Exemption], List[models.ExemptionTypedDict]]
        ] = UNSET,
        related_to: OptionalNullable[str] = UNSET,
        secondary_external_id: OptionalNullable[str] = UNSET,
        secondary_source: OptionalNullable[str] = UNSET,
        external_friendly_id: OptionalNullable[str] = UNSET,
        total_tax_amount_imported: Optional[
            Union[
                models.TransactionCreateTotalTaxAmountImported,
                models.TransactionCreateTotalTaxAmountImportedTypedDict,
            ]
        ] = None,
        tax_rate_imported: Optional[
            Union[
                models.TransactionCreateTaxRateImported,
                models.TransactionCreateTaxRateImportedTypedDict,
            ]
        ] = None,
        total_tax_amount_calculated: Optional[
            Union[
                models.TransactionCreateTotalTaxAmountCalculated,
                models.TransactionCreateTotalTaxAmountCalculatedTypedDict,
            ]
        ] = None,
        tax_rate_calculated: Optional[
            Union[
                models.TransactionCreateTaxRateCalculated,
                models.TransactionCreateTaxRateCalculatedTypedDict,
            ]
        ] = None,
        total_tax_liability_amount: Optional[
            Union[
                models.TransactionCreateTotalTaxLiabilityAmount,
                models.TransactionCreateTotalTaxLiabilityAmountTypedDict,
            ]
        ] = None,
        tax_liability_source: OptionalNullable[models.TaxLiabilitySourceEnum] = UNSET,
        taxable_amount: Optional[
            Union[
                models.TransactionCreateTaxableAmount,
                models.TransactionCreateTaxableAmountTypedDict,
            ]
        ] = None,
        currency: Optional[models.CurrencyEnum] = None,
        locked: Optional[bool] = False,
        source: Optional[models.SourceEnum] = None,
        connection_id: OptionalNullable[str] = UNSET,
        filing_id: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        address_status: Optional[models.AddressStatus] = None,
        processing_status: Optional[models.ProcessingStatusEnum] = None,
        destination_currency: OptionalNullable[models.CurrencyEnum] = UNSET,
        converted_total_amount: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalAmount,
                models.TransactionCreateConvertedTotalAmountTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_amount_imported: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalTaxAmountImported,
                models.TransactionCreateConvertedTotalTaxAmountImportedTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_amount_calculated: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalTaxAmountCalculated,
                models.TransactionCreateConvertedTotalTaxAmountCalculatedTypedDict,
            ]
        ] = UNSET,
        conversion_rate: OptionalNullable[
            Union[
                models.TransactionCreateConversionRate,
                models.TransactionCreateConversionRateTypedDict,
            ]
        ] = UNSET,
        converted_taxable_amount: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTaxableAmount,
                models.TransactionCreateConvertedTaxableAmountTypedDict,
            ]
        ] = UNSET,
        converted_total_discount: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalDiscount,
                models.TransactionCreateConvertedTotalDiscountTypedDict,
            ]
        ] = UNSET,
        converted_subtotal: OptionalNullable[
            Union[
                models.TransactionCreateConvertedSubtotal,
                models.TransactionCreateConvertedSubtotalTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_liability_amount: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalTaxLiabilityAmount,
                models.TransactionCreateConvertedTotalTaxLiabilityAmountTypedDict,
            ]
        ] = UNSET,
        customer: OptionalNullable[
            Union[models.CustomerCreate, models.CustomerCreateTypedDict]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Create Transaction By Customer Id

        Create a new transaction for a specific customer.

        :param security:
        :param customer_id_param:
        :param x_organization_id: The unique identifier for the organization making the request
        :param organization_id: Unique identifier of the organization.
        :param external_id: External identifier of the transaction.
        :param date_: Transaction date and time
        :param addresses:
        :param transaction_items:
        :param requires_exemption: Indicates if transaction requires tax exemption.
        :param shop_date: Transaction date in the shop's local timezone
        :param shop_date_tz: Timezone of the shop
        :param status:
        :param description: Description of the transaction.
        :param refund_status: Status of refund, if applicable
        :param total_amount: Total amount of the transaction.
        :param customer_id: Unique identifier of the customer.
        :param marketplace: Indicates if transaction is marketplace-based.
        :param exempt: Exemption status (e.g., NOT_EXEMPT)
        :param exemptions: List of exemptions applied (if any).
        :param related_to: Related transaction identifier.
        :param secondary_external_id: Secondary External Identifier.
        :param secondary_source: Secondary source information
        :param external_friendly_id: Friendly identifier of the original item.
        :param total_tax_amount_imported: Imported tax amount.
        :param tax_rate_imported: Imported tax rate.
        :param total_tax_amount_calculated: Calculated tax amount.
        :param tax_rate_calculated: Calculated tax rate.
        :param total_tax_liability_amount: Total tax liability amount.
        :param tax_liability_source: Source of tax liability.
        :param taxable_amount: Taxable amount.
        :param currency:
        :param locked: Transaction lock status.
        :param source:
        :param connection_id: Connection Identifier
        :param filing_id: Filing identifier.
        :param city: City of the transaction address.
        :param county: County of the transaction address.
        :param state: State of the transaction address.
        :param country: Country code (ISO Alpha-2).
        :param postal_code: Postal code of the transaction.
        :param tax_id: Tax ID associated with the transaction
        :param address_status:
        :param processing_status: Our transaction state, used to determine when/if a transaction needs additional processing.
        :param destination_currency: Destination currency code (ISO 4217, e.g., USD)
        :param converted_total_amount: Converted total amount.
        :param converted_total_tax_amount_imported: Converted imported tax amount.
        :param converted_total_tax_amount_calculated: Converted calculated tax amount.
        :param conversion_rate: Currency conversion rate.
        :param converted_taxable_amount: Converted taxable amount.
        :param converted_total_discount: Converted total discount amount.
        :param converted_subtotal: Converted subtotal amount.
        :param converted_total_tax_liability_amount: Converted total tax liability amount.
        :param customer:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateTransactionByCustomerIDV1CustomersCustomerIDTransactionsPostRequest(
            customer_id_param=customer_id_param,
            x_organization_id=x_organization_id,
            transaction_create=models.TransactionCreate(
                requires_exemption=utils.get_pydantic_model(
                    requires_exemption, OptionalNullable[models.ExemptionRequired]
                ),
                organization_id=organization_id,
                external_id=external_id,
                date_=date_,
                shop_date=shop_date,
                shop_date_tz=shop_date_tz,
                status=status,
                description=description,
                refund_status=refund_status,
                total_amount=total_amount,
                customer_id=customer_id,
                marketplace=marketplace,
                exempt=exempt,
                exemptions=utils.get_pydantic_model(
                    exemptions, OptionalNullable[List[models.Exemption]]
                ),
                related_to=related_to,
                secondary_external_id=secondary_external_id,
                secondary_source=secondary_source,
                external_friendly_id=external_friendly_id,
                total_tax_amount_imported=total_tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                total_tax_amount_calculated=total_tax_amount_calculated,
                tax_rate_calculated=tax_rate_calculated,
                total_tax_liability_amount=total_tax_liability_amount,
                tax_liability_source=tax_liability_source,
                taxable_amount=taxable_amount,
                currency=currency,
                locked=locked,
                source=source,
                connection_id=connection_id,
                filing_id=filing_id,
                city=city,
                county=county,
                state=state,
                country=country,
                postal_code=postal_code,
                tax_id=tax_id,
                address_status=address_status,
                processing_status=processing_status,
                destination_currency=destination_currency,
                converted_total_amount=converted_total_amount,
                converted_total_tax_amount_imported=converted_total_tax_amount_imported,
                converted_total_tax_amount_calculated=converted_total_tax_amount_calculated,
                conversion_rate=conversion_rate,
                converted_taxable_amount=converted_taxable_amount,
                converted_total_discount=converted_total_discount,
                converted_subtotal=converted_subtotal,
                converted_total_tax_liability_amount=converted_total_tax_liability_amount,
                addresses=utils.get_pydantic_model(
                    addresses, models.TransactionCreateAddresses
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.TransactionItemCreateUpdate]
                ),
                customer=utils.get_pydantic_model(
                    customer, OptionalNullable[models.CustomerCreate]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/customers/{customer_id}/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.CreateTransactionByCustomerIDV1CustomersCustomerIDTransactionsPostSecurity,
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.transaction_create,
                False,
                False,
                "json",
                models.TransactionCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_transaction_by_customer_id_v1_customers__customer_id__transactions_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_transaction_async(
        self,
        *,
        security: Union[
            models.CreateTransactionByCustomerIDV1CustomersCustomerIDTransactionsPostSecurity,
            models.CreateTransactionByCustomerIDV1CustomersCustomerIDTransactionsPostSecurityTypedDict,
        ],
        customer_id_param: str,
        x_organization_id: Nullable[str],
        organization_id: str,
        external_id: str,
        date_: datetime,
        addresses: Union[
            models.TransactionCreateAddresses,
            models.TransactionCreateAddressesTypedDict,
        ],
        transaction_items: Union[
            List[models.TransactionItemCreateUpdate],
            List[models.TransactionItemCreateUpdateTypedDict],
        ],
        requires_exemption: OptionalNullable[
            Union[models.ExemptionRequired, models.ExemptionRequiredTypedDict]
        ] = UNSET,
        shop_date: OptionalNullable[date] = UNSET,
        shop_date_tz: OptionalNullable[str] = UNSET,
        status: Optional[models.TransactionStatusEnum] = None,
        description: OptionalNullable[str] = UNSET,
        refund_status: OptionalNullable[models.TransactionRefundStatus] = UNSET,
        total_amount: Optional[
            Union[
                models.TransactionCreateTotalAmount,
                models.TransactionCreateTotalAmountTypedDict,
            ]
        ] = None,
        customer_id: OptionalNullable[str] = UNSET,
        marketplace: OptionalNullable[bool] = UNSET,
        exempt: OptionalNullable[models.TransactionExemptStatusEnum] = UNSET,
        exemptions: OptionalNullable[
            Union[List[models.Exemption], List[models.ExemptionTypedDict]]
        ] = UNSET,
        related_to: OptionalNullable[str] = UNSET,
        secondary_external_id: OptionalNullable[str] = UNSET,
        secondary_source: OptionalNullable[str] = UNSET,
        external_friendly_id: OptionalNullable[str] = UNSET,
        total_tax_amount_imported: Optional[
            Union[
                models.TransactionCreateTotalTaxAmountImported,
                models.TransactionCreateTotalTaxAmountImportedTypedDict,
            ]
        ] = None,
        tax_rate_imported: Optional[
            Union[
                models.TransactionCreateTaxRateImported,
                models.TransactionCreateTaxRateImportedTypedDict,
            ]
        ] = None,
        total_tax_amount_calculated: Optional[
            Union[
                models.TransactionCreateTotalTaxAmountCalculated,
                models.TransactionCreateTotalTaxAmountCalculatedTypedDict,
            ]
        ] = None,
        tax_rate_calculated: Optional[
            Union[
                models.TransactionCreateTaxRateCalculated,
                models.TransactionCreateTaxRateCalculatedTypedDict,
            ]
        ] = None,
        total_tax_liability_amount: Optional[
            Union[
                models.TransactionCreateTotalTaxLiabilityAmount,
                models.TransactionCreateTotalTaxLiabilityAmountTypedDict,
            ]
        ] = None,
        tax_liability_source: OptionalNullable[models.TaxLiabilitySourceEnum] = UNSET,
        taxable_amount: Optional[
            Union[
                models.TransactionCreateTaxableAmount,
                models.TransactionCreateTaxableAmountTypedDict,
            ]
        ] = None,
        currency: Optional[models.CurrencyEnum] = None,
        locked: Optional[bool] = False,
        source: Optional[models.SourceEnum] = None,
        connection_id: OptionalNullable[str] = UNSET,
        filing_id: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        address_status: Optional[models.AddressStatus] = None,
        processing_status: Optional[models.ProcessingStatusEnum] = None,
        destination_currency: OptionalNullable[models.CurrencyEnum] = UNSET,
        converted_total_amount: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalAmount,
                models.TransactionCreateConvertedTotalAmountTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_amount_imported: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalTaxAmountImported,
                models.TransactionCreateConvertedTotalTaxAmountImportedTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_amount_calculated: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalTaxAmountCalculated,
                models.TransactionCreateConvertedTotalTaxAmountCalculatedTypedDict,
            ]
        ] = UNSET,
        conversion_rate: OptionalNullable[
            Union[
                models.TransactionCreateConversionRate,
                models.TransactionCreateConversionRateTypedDict,
            ]
        ] = UNSET,
        converted_taxable_amount: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTaxableAmount,
                models.TransactionCreateConvertedTaxableAmountTypedDict,
            ]
        ] = UNSET,
        converted_total_discount: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalDiscount,
                models.TransactionCreateConvertedTotalDiscountTypedDict,
            ]
        ] = UNSET,
        converted_subtotal: OptionalNullable[
            Union[
                models.TransactionCreateConvertedSubtotal,
                models.TransactionCreateConvertedSubtotalTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_liability_amount: OptionalNullable[
            Union[
                models.TransactionCreateConvertedTotalTaxLiabilityAmount,
                models.TransactionCreateConvertedTotalTaxLiabilityAmountTypedDict,
            ]
        ] = UNSET,
        customer: OptionalNullable[
            Union[models.CustomerCreate, models.CustomerCreateTypedDict]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Create Transaction By Customer Id

        Create a new transaction for a specific customer.

        :param security:
        :param customer_id_param:
        :param x_organization_id: The unique identifier for the organization making the request
        :param organization_id: Unique identifier of the organization.
        :param external_id: External identifier of the transaction.
        :param date_: Transaction date and time
        :param addresses:
        :param transaction_items:
        :param requires_exemption: Indicates if transaction requires tax exemption.
        :param shop_date: Transaction date in the shop's local timezone
        :param shop_date_tz: Timezone of the shop
        :param status:
        :param description: Description of the transaction.
        :param refund_status: Status of refund, if applicable
        :param total_amount: Total amount of the transaction.
        :param customer_id: Unique identifier of the customer.
        :param marketplace: Indicates if transaction is marketplace-based.
        :param exempt: Exemption status (e.g., NOT_EXEMPT)
        :param exemptions: List of exemptions applied (if any).
        :param related_to: Related transaction identifier.
        :param secondary_external_id: Secondary External Identifier.
        :param secondary_source: Secondary source information
        :param external_friendly_id: Friendly identifier of the original item.
        :param total_tax_amount_imported: Imported tax amount.
        :param tax_rate_imported: Imported tax rate.
        :param total_tax_amount_calculated: Calculated tax amount.
        :param tax_rate_calculated: Calculated tax rate.
        :param total_tax_liability_amount: Total tax liability amount.
        :param tax_liability_source: Source of tax liability.
        :param taxable_amount: Taxable amount.
        :param currency:
        :param locked: Transaction lock status.
        :param source:
        :param connection_id: Connection Identifier
        :param filing_id: Filing identifier.
        :param city: City of the transaction address.
        :param county: County of the transaction address.
        :param state: State of the transaction address.
        :param country: Country code (ISO Alpha-2).
        :param postal_code: Postal code of the transaction.
        :param tax_id: Tax ID associated with the transaction
        :param address_status:
        :param processing_status: Our transaction state, used to determine when/if a transaction needs additional processing.
        :param destination_currency: Destination currency code (ISO 4217, e.g., USD)
        :param converted_total_amount: Converted total amount.
        :param converted_total_tax_amount_imported: Converted imported tax amount.
        :param converted_total_tax_amount_calculated: Converted calculated tax amount.
        :param conversion_rate: Currency conversion rate.
        :param converted_taxable_amount: Converted taxable amount.
        :param converted_total_discount: Converted total discount amount.
        :param converted_subtotal: Converted subtotal amount.
        :param converted_total_tax_liability_amount: Converted total tax liability amount.
        :param customer:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateTransactionByCustomerIDV1CustomersCustomerIDTransactionsPostRequest(
            customer_id_param=customer_id_param,
            x_organization_id=x_organization_id,
            transaction_create=models.TransactionCreate(
                requires_exemption=utils.get_pydantic_model(
                    requires_exemption, OptionalNullable[models.ExemptionRequired]
                ),
                organization_id=organization_id,
                external_id=external_id,
                date_=date_,
                shop_date=shop_date,
                shop_date_tz=shop_date_tz,
                status=status,
                description=description,
                refund_status=refund_status,
                total_amount=total_amount,
                customer_id=customer_id,
                marketplace=marketplace,
                exempt=exempt,
                exemptions=utils.get_pydantic_model(
                    exemptions, OptionalNullable[List[models.Exemption]]
                ),
                related_to=related_to,
                secondary_external_id=secondary_external_id,
                secondary_source=secondary_source,
                external_friendly_id=external_friendly_id,
                total_tax_amount_imported=total_tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                total_tax_amount_calculated=total_tax_amount_calculated,
                tax_rate_calculated=tax_rate_calculated,
                total_tax_liability_amount=total_tax_liability_amount,
                tax_liability_source=tax_liability_source,
                taxable_amount=taxable_amount,
                currency=currency,
                locked=locked,
                source=source,
                connection_id=connection_id,
                filing_id=filing_id,
                city=city,
                county=county,
                state=state,
                country=country,
                postal_code=postal_code,
                tax_id=tax_id,
                address_status=address_status,
                processing_status=processing_status,
                destination_currency=destination_currency,
                converted_total_amount=converted_total_amount,
                converted_total_tax_amount_imported=converted_total_tax_amount_imported,
                converted_total_tax_amount_calculated=converted_total_tax_amount_calculated,
                conversion_rate=conversion_rate,
                converted_taxable_amount=converted_taxable_amount,
                converted_total_discount=converted_total_discount,
                converted_subtotal=converted_subtotal,
                converted_total_tax_liability_amount=converted_total_tax_liability_amount,
                addresses=utils.get_pydantic_model(
                    addresses, models.TransactionCreateAddresses
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.TransactionItemCreateUpdate]
                ),
                customer=utils.get_pydantic_model(
                    customer, OptionalNullable[models.CustomerCreate]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/customers/{customer_id}/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.CreateTransactionByCustomerIDV1CustomersCustomerIDTransactionsPostSecurity,
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.transaction_create,
                False,
                False,
                "json",
                models.TransactionCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_transaction_by_customer_id_v1_customers__customer_id__transactions_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)
