"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import date
from kintsugi_tax_platform_sdk import errors, models, utils
from kintsugi_tax_platform_sdk._hooks import HookContext
from kintsugi_tax_platform_sdk.types import Nullable, OptionalNullable, UNSET
from kintsugi_tax_platform_sdk.utils.unmarshal_json_response import (
    unmarshal_json_response,
)
from typing import Any, List, Mapping, Optional, Union


class Exemptions(BaseSDK):
    def list(
        self,
        *,
        security: Union[
            models.GetExemptionsV1ExemptionsGetSecurity,
            models.GetExemptionsV1ExemptionsGetSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        search_query: OptionalNullable[str] = UNSET,
        status_in: OptionalNullable[str] = UNSET,
        country_code: OptionalNullable[
            Union[List[models.CountryCode], List[models.CountryCodeTypedDict]]
        ] = UNSET,
        jurisdiction: OptionalNullable[str] = UNSET,
        start_date: OptionalNullable[date] = UNSET,
        end_date: OptionalNullable[date] = UNSET,
        customer_id: OptionalNullable[str] = UNSET,
        transaction_id: OptionalNullable[str] = UNSET,
        order_by: OptionalNullable[str] = UNSET,
        page: Optional[int] = 1,
        size: Optional[int] = 50,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FastapiPaginationDefaultPageExemptionRead2:
        r"""Get Exemptions

        Retrieve a list of exemptions based on filters.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param search_query: Search term to filter exemptions by exemption ID, customer name, or customer email
        :param status_in: Filter exemptions by their status
        :param country_code: Country code in ISO 3166-1 alpha-2 format
        :param jurisdiction: Jurisdiction identifier
        :param start_date: Start date for filtering exemptions
        :param end_date: End date for filtering exemptions
        :param customer_id: Customer ID to filter exemptions
        :param transaction_id: Transaction ID to filter exemptions
        :param order_by: Fields to sort by (comma-separated)
        :param page: Page number
        :param size: Page size
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetExemptionsV1ExemptionsGetRequest(
            search_query=search_query,
            status_in=status_in,
            country_code=country_code,
            jurisdiction=jurisdiction,
            start_date=start_date,
            end_date=end_date,
            customer_id=customer_id,
            transaction_id=transaction_id,
            order_by=order_by,
            page=page,
            size=size,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/exemptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetExemptionsV1ExemptionsGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_exemptions_v1_exemptions_get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.FastapiPaginationDefaultPageExemptionRead2, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        security: Union[
            models.GetExemptionsV1ExemptionsGetSecurity,
            models.GetExemptionsV1ExemptionsGetSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        search_query: OptionalNullable[str] = UNSET,
        status_in: OptionalNullable[str] = UNSET,
        country_code: OptionalNullable[
            Union[List[models.CountryCode], List[models.CountryCodeTypedDict]]
        ] = UNSET,
        jurisdiction: OptionalNullable[str] = UNSET,
        start_date: OptionalNullable[date] = UNSET,
        end_date: OptionalNullable[date] = UNSET,
        customer_id: OptionalNullable[str] = UNSET,
        transaction_id: OptionalNullable[str] = UNSET,
        order_by: OptionalNullable[str] = UNSET,
        page: Optional[int] = 1,
        size: Optional[int] = 50,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FastapiPaginationDefaultPageExemptionRead2:
        r"""Get Exemptions

        Retrieve a list of exemptions based on filters.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param search_query: Search term to filter exemptions by exemption ID, customer name, or customer email
        :param status_in: Filter exemptions by their status
        :param country_code: Country code in ISO 3166-1 alpha-2 format
        :param jurisdiction: Jurisdiction identifier
        :param start_date: Start date for filtering exemptions
        :param end_date: End date for filtering exemptions
        :param customer_id: Customer ID to filter exemptions
        :param transaction_id: Transaction ID to filter exemptions
        :param order_by: Fields to sort by (comma-separated)
        :param page: Page number
        :param size: Page size
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetExemptionsV1ExemptionsGetRequest(
            search_query=search_query,
            status_in=status_in,
            country_code=country_code,
            jurisdiction=jurisdiction,
            start_date=start_date,
            end_date=end_date,
            customer_id=customer_id,
            transaction_id=transaction_id,
            order_by=order_by,
            page=page,
            size=size,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/exemptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetExemptionsV1ExemptionsGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_exemptions_v1_exemptions_get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.FastapiPaginationDefaultPageExemptionRead2, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        security: Union[
            models.CreateExemptionV1ExemptionsPostSecurity,
            models.CreateExemptionV1ExemptionsPostSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        exemption_type: models.ExemptionType,
        start_date: date,
        customer_id: str,
        fein: str,
        sales_tax_id: str,
        status: models.ExemptionStatus,
        jurisdiction: OptionalNullable[str] = UNSET,
        country_code: OptionalNullable[models.CountryCodeEnum] = UNSET,
        end_date: OptionalNullable[date] = UNSET,
        transaction_id: OptionalNullable[str] = UNSET,
        reseller: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.BackendSrcExemptionsSerializersExemptionRead:
        r"""Create Exemption

        The Create Exemption API allows you to create a new exemption record.
        This includes defining details such as exemption type, jurisdiction,
        Country, State, validity dates, etc.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param exemption_type:
        :param start_date: Start date for the exemption validity period (YYYY-MM-DD format)
        :param customer_id: Unique identifier for the customer associated with the exemption
        :param fein: Federal Employer Identification Number
        :param sales_tax_id: Sales tax ID for the exemption
        :param status:
        :param jurisdiction: The jurisdiction identifier for the exemption
        :param country_code: Country code in ISO 3166-1 alpha-2 format (e.g., 'US')
        :param end_date: End date for the exemption validity period (YYYY-MM-DD format)
        :param transaction_id: Unique identifier for the transaction, if applicable
        :param reseller: Indicates whether the exemption is for a reseller
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateExemptionV1ExemptionsPostRequest(
            x_organization_id=x_organization_id,
            exemption_create=models.ExemptionCreate(
                exemption_type=exemption_type,
                jurisdiction=jurisdiction,
                country_code=country_code,
                start_date=start_date,
                end_date=end_date,
                customer_id=customer_id,
                transaction_id=transaction_id,
                reseller=reseller,
                fein=fein,
                sales_tax_id=sales_tax_id,
                status=status,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/exemptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CreateExemptionV1ExemptionsPostSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.exemption_create, False, False, "json", models.ExemptionCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_exemption_v1_exemptions_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.BackendSrcExemptionsSerializersExemptionRead, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        security: Union[
            models.CreateExemptionV1ExemptionsPostSecurity,
            models.CreateExemptionV1ExemptionsPostSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        exemption_type: models.ExemptionType,
        start_date: date,
        customer_id: str,
        fein: str,
        sales_tax_id: str,
        status: models.ExemptionStatus,
        jurisdiction: OptionalNullable[str] = UNSET,
        country_code: OptionalNullable[models.CountryCodeEnum] = UNSET,
        end_date: OptionalNullable[date] = UNSET,
        transaction_id: OptionalNullable[str] = UNSET,
        reseller: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.BackendSrcExemptionsSerializersExemptionRead:
        r"""Create Exemption

        The Create Exemption API allows you to create a new exemption record.
        This includes defining details such as exemption type, jurisdiction,
        Country, State, validity dates, etc.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param exemption_type:
        :param start_date: Start date for the exemption validity period (YYYY-MM-DD format)
        :param customer_id: Unique identifier for the customer associated with the exemption
        :param fein: Federal Employer Identification Number
        :param sales_tax_id: Sales tax ID for the exemption
        :param status:
        :param jurisdiction: The jurisdiction identifier for the exemption
        :param country_code: Country code in ISO 3166-1 alpha-2 format (e.g., 'US')
        :param end_date: End date for the exemption validity period (YYYY-MM-DD format)
        :param transaction_id: Unique identifier for the transaction, if applicable
        :param reseller: Indicates whether the exemption is for a reseller
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateExemptionV1ExemptionsPostRequest(
            x_organization_id=x_organization_id,
            exemption_create=models.ExemptionCreate(
                exemption_type=exemption_type,
                jurisdiction=jurisdiction,
                country_code=country_code,
                start_date=start_date,
                end_date=end_date,
                customer_id=customer_id,
                transaction_id=transaction_id,
                reseller=reseller,
                fein=fein,
                sales_tax_id=sales_tax_id,
                status=status,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/exemptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CreateExemptionV1ExemptionsPostSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.exemption_create, False, False, "json", models.ExemptionCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_exemption_v1_exemptions_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.BackendSrcExemptionsSerializersExemptionRead, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        security: Union[
            models.GetExemptionByIDV1ExemptionsExemptionIDGetSecurity,
            models.GetExemptionByIDV1ExemptionsExemptionIDGetSecurityTypedDict,
        ],
        exemption_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.BackendSrcExemptionsModelsExemptionRead:
        r"""Get Exemption By Id

        The Get Exemption By ID API retrieves a specific exemption record by
        its unique ID. This API is useful for retrieving detailed information
        about a particular exemption, including its associated
        customer, organisation id, status, etc.

        :param security:
        :param exemption_id: The unique identifier for the exemption being retrieved.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetExemptionByIDV1ExemptionsExemptionIDGetRequest(
            exemption_id=exemption_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/exemptions/{exemption_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetExemptionByIDV1ExemptionsExemptionIDGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_exemption_by_id_v1_exemptions__exemption_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.BackendSrcExemptionsModelsExemptionRead, http_res
            )
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        security: Union[
            models.GetExemptionByIDV1ExemptionsExemptionIDGetSecurity,
            models.GetExemptionByIDV1ExemptionsExemptionIDGetSecurityTypedDict,
        ],
        exemption_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.BackendSrcExemptionsModelsExemptionRead:
        r"""Get Exemption By Id

        The Get Exemption By ID API retrieves a specific exemption record by
        its unique ID. This API is useful for retrieving detailed information
        about a particular exemption, including its associated
        customer, organisation id, status, etc.

        :param security:
        :param exemption_id: The unique identifier for the exemption being retrieved.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetExemptionByIDV1ExemptionsExemptionIDGetRequest(
            exemption_id=exemption_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/exemptions/{exemption_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetExemptionByIDV1ExemptionsExemptionIDGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_exemption_by_id_v1_exemptions__exemption_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.BackendSrcExemptionsModelsExemptionRead, http_res
            )
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def upload_certificate(
        self,
        *,
        security: Union[
            models.UploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPostSecurity,
            models.UploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPostSecurityTypedDict,
        ],
        exemption_id: str,
        x_organization_id: Nullable[str],
        file: Union[models.File, models.FileTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AttachmentRead:
        r"""Upload Exemption Certificate

        The Upload Exemption Certificate API allows you
        to upload a file attachment (e.g., exemption certificate) for a specific exemption.
        This is primarily used to associate supporting documents with an exemption record
        to ensure compliance and facilitate verification.

        :param security:
        :param exemption_id: The unique identifier for the exemption to which the attachment will be associated.
        :param x_organization_id: The unique identifier for the organization making the request
        :param file: The file to be uploaded. Supported format: PDF. Max size: 10 MB.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPostRequest(
            exemption_id=exemption_id,
            x_organization_id=x_organization_id,
            body_upload_exemption_certificate_v1_exemptions_exemption_id_attachments_post=models.BodyUploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPost(
                file=utils.get_pydantic_model(file, models.File),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/exemptions/{exemption_id}/attachments",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.UploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPostSecurity,
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body_upload_exemption_certificate_v1_exemptions_exemption_id_attachments_post,
                False,
                False,
                "multipart",
                models.BodyUploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPost,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="upload_exemption_certificate_v1_exemptions__exemption_id__attachments_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.AttachmentRead, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def upload_certificate_async(
        self,
        *,
        security: Union[
            models.UploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPostSecurity,
            models.UploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPostSecurityTypedDict,
        ],
        exemption_id: str,
        x_organization_id: Nullable[str],
        file: Union[models.File, models.FileTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AttachmentRead:
        r"""Upload Exemption Certificate

        The Upload Exemption Certificate API allows you
        to upload a file attachment (e.g., exemption certificate) for a specific exemption.
        This is primarily used to associate supporting documents with an exemption record
        to ensure compliance and facilitate verification.

        :param security:
        :param exemption_id: The unique identifier for the exemption to which the attachment will be associated.
        :param x_organization_id: The unique identifier for the organization making the request
        :param file: The file to be uploaded. Supported format: PDF. Max size: 10 MB.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPostRequest(
            exemption_id=exemption_id,
            x_organization_id=x_organization_id,
            body_upload_exemption_certificate_v1_exemptions_exemption_id_attachments_post=models.BodyUploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPost(
                file=utils.get_pydantic_model(file, models.File),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/exemptions/{exemption_id}/attachments",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.UploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPostSecurity,
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body_upload_exemption_certificate_v1_exemptions_exemption_id_attachments_post,
                False,
                False,
                "multipart",
                models.BodyUploadExemptionCertificateV1ExemptionsExemptionIDAttachmentsPost,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="upload_exemption_certificate_v1_exemptions__exemption_id__attachments_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.AttachmentRead, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_attachments(
        self,
        *,
        security: Union[
            models.GetAttachmentsForExemptionV1ExemptionsExemptionIDAttachmentsGetSecurity,
            models.GetAttachmentsForExemptionV1ExemptionsExemptionIDAttachmentsGetSecurityTypedDict,
        ],
        exemption_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.AttachmentRead]:
        r"""Get Attachments For Exemption

        The Get Attachments for Exemption API retrieves all
        attachments associated with a specific exemption.
        This is used to view and manage supporting documents
        like exemption certificates uploaded for a particular exemption record.

        :param security:
        :param exemption_id: The unique identifier for the exemption         whose attachments are being retrieved.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetAttachmentsForExemptionV1ExemptionsExemptionIDAttachmentsGetRequest(
            exemption_id=exemption_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/exemptions/{exemption_id}/attachments",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetAttachmentsForExemptionV1ExemptionsExemptionIDAttachmentsGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_attachments_for_exemption_v1_exemptions__exemption_id__attachments_get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.AttachmentRead], http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_attachments_async(
        self,
        *,
        security: Union[
            models.GetAttachmentsForExemptionV1ExemptionsExemptionIDAttachmentsGetSecurity,
            models.GetAttachmentsForExemptionV1ExemptionsExemptionIDAttachmentsGetSecurityTypedDict,
        ],
        exemption_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.AttachmentRead]:
        r"""Get Attachments For Exemption

        The Get Attachments for Exemption API retrieves all
        attachments associated with a specific exemption.
        This is used to view and manage supporting documents
        like exemption certificates uploaded for a particular exemption record.

        :param security:
        :param exemption_id: The unique identifier for the exemption         whose attachments are being retrieved.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetAttachmentsForExemptionV1ExemptionsExemptionIDAttachmentsGetRequest(
            exemption_id=exemption_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/exemptions/{exemption_id}/attachments",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetAttachmentsForExemptionV1ExemptionsExemptionIDAttachmentsGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_attachments_for_exemption_v1_exemptions__exemption_id__attachments_get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.AttachmentRead], http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcExemptionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcExemptionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)
