"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .currencyenum import CurrencyEnum
from .customerbase import CustomerBase, CustomerBaseTypedDict
from .sourceenum import SourceEnum
from .transactionitemestimateresponse import (
    TransactionItemEstimateResponse,
    TransactionItemEstimateResponseTypedDict,
)
from datetime import datetime
from enum import Enum
from kintsugi_tax_platform_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TransactionEstimateResponseType(str, Enum):
    r"""Type of the address. Must be either
    SHIP_TO or BILL_TO.
    """

    SHIP_TO = "SHIP_TO"
    BILL_TO = "BILL_TO"


class TransactionEstimateResponseAddressTypedDict(TypedDict):
    type: TransactionEstimateResponseType
    r"""Type of the address. Must be either
    SHIP_TO or BILL_TO.
    """
    state: str
    r"""State or province of the address."""
    postal_code: str
    r"""Postal code of the address."""
    country: str
    r"""Country of the address."""
    phone: NotRequired[str]
    r"""Phone number associated with the customer."""
    street_1: NotRequired[str]
    r"""Primary street address of the customer."""
    street_2: NotRequired[str]
    r"""Additional street address details, such as an apartment or suite number."""
    city: NotRequired[str]
    r"""City where the customer resides."""
    county: NotRequired[str]
    r"""County or district of the customer."""
    full_address: NotRequired[str]
    r"""Complete address string of the customer, which can be used as an alternative to individual fields."""
    status: NotRequired[str]
    r"""Status of the address. Deprecated and ignored."""
    enriched_fields: NotRequired[str]
    r"""Additional enriched fields related to the address."""


class TransactionEstimateResponseAddress(BaseModel):
    type: TransactionEstimateResponseType
    r"""Type of the address. Must be either
    SHIP_TO or BILL_TO.
    """

    state: str
    r"""State or province of the address."""

    postal_code: str
    r"""Postal code of the address."""

    country: str
    r"""Country of the address."""

    phone: Optional[str] = None
    r"""Phone number associated with the customer."""

    street_1: Optional[str] = None
    r"""Primary street address of the customer."""

    street_2: Optional[str] = None
    r"""Additional street address details, such as an apartment or suite number."""

    city: Optional[str] = None
    r"""City where the customer resides."""

    county: Optional[str] = None
    r"""County or district of the customer."""

    full_address: Optional[str] = None
    r"""Complete address string of the customer, which can be used as an alternative to individual fields."""

    status: Annotated[
        Optional[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""Status of the address. Deprecated and ignored."""

    enriched_fields: Optional[str] = None
    r"""Additional enriched fields related to the address."""


class TransactionEstimateResponseTypedDict(TypedDict):
    date_: datetime
    r"""The date of the transaction in ISO 8601 format (e.g., 2025-01-25T12:00:00Z)."""
    external_id: str
    r"""Unique identifier of this transaction in the source system."""
    currency: CurrencyEnum
    transaction_items: List[TransactionItemEstimateResponseTypedDict]
    addresses: List[TransactionEstimateResponseAddressTypedDict]
    r"""List of addresses related to the transaction. At least one BILL_TO or SHIP_TO address must be provided. The address will be validated during estimation, and the transaction may be rejected if the address does not pass validation. The SHIP_TO will be preferred to use for determining tax liability. **Deprecated:** Use of `address.status` in estimate api is ignored and will be removed in the future status will be considered UNVERIFIED by default and always validated"""
    total_amount: NotRequired[str]
    r"""Total amount of the transaction."""
    description: NotRequired[Nullable[str]]
    r"""An optional description of the transaction."""
    source: NotRequired[Nullable[SourceEnum]]
    r"""While currently not used, it may be used in the future to determine taxability. The source of the transaction (e.g., OTHER)."""
    marketplace: NotRequired[Nullable[bool]]
    r"""Indicates if the transaction involves a marketplace."""
    customer: NotRequired[Nullable[CustomerBaseTypedDict]]
    r"""Details about the customer. If the customer is not found, it will be ignored."""
    total_tax_amount_calculated: NotRequired[str]
    r"""The total amount of tax determined for the transaction."""
    taxable_amount: NotRequired[str]
    r"""The taxable amount for the transaction."""
    tax_rate_calculated: NotRequired[str]
    r"""The calculated tax rate for the transaction."""
    nexus_met: NotRequired[bool]
    r"""Indicates if nexus was met for the transaction.**Deprecated:** Use `has_active_registration` instead."""
    has_active_registration: NotRequired[bool]
    r"""Indicates if there is an active registration for the transaction."""


class TransactionEstimateResponse(BaseModel):
    date_: Annotated[datetime, pydantic.Field(alias="date")]
    r"""The date of the transaction in ISO 8601 format (e.g., 2025-01-25T12:00:00Z)."""

    external_id: str
    r"""Unique identifier of this transaction in the source system."""

    currency: CurrencyEnum

    transaction_items: List[TransactionItemEstimateResponse]

    addresses: List[TransactionEstimateResponseAddress]
    r"""List of addresses related to the transaction. At least one BILL_TO or SHIP_TO address must be provided. The address will be validated during estimation, and the transaction may be rejected if the address does not pass validation. The SHIP_TO will be preferred to use for determining tax liability. **Deprecated:** Use of `address.status` in estimate api is ignored and will be removed in the future status will be considered UNVERIFIED by default and always validated"""

    total_amount: Optional[str] = "0.0"
    r"""Total amount of the transaction."""

    description: OptionalNullable[str] = UNSET
    r"""An optional description of the transaction."""

    source: Annotated[
        OptionalNullable[SourceEnum],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""While currently not used, it may be used in the future to determine taxability. The source of the transaction (e.g., OTHER)."""

    marketplace: OptionalNullable[bool] = UNSET
    r"""Indicates if the transaction involves a marketplace."""

    customer: OptionalNullable[CustomerBase] = UNSET
    r"""Details about the customer. If the customer is not found, it will be ignored."""

    total_tax_amount_calculated: Optional[str] = "0.00"
    r"""The total amount of tax determined for the transaction."""

    taxable_amount: Optional[str] = "0.00"
    r"""The taxable amount for the transaction."""

    tax_rate_calculated: Optional[str] = "0.00"
    r"""The calculated tax rate for the transaction."""

    nexus_met: Annotated[
        Optional[bool],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = False
    r"""Indicates if nexus was met for the transaction.**Deprecated:** Use `has_active_registration` instead."""

    has_active_registration: Optional[bool] = False
    r"""Indicates if there is an active registration for the transaction."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "total_amount",
            "description",
            "source",
            "marketplace",
            "customer",
            "total_tax_amount_calculated",
            "taxable_amount",
            "tax_rate_calculated",
            "nexus_met",
            "has_active_registration",
        ]
        nullable_fields = ["description", "source", "marketplace", "customer"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
