"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import date, datetime
from kintsugi_tax_platform_sdk import errors, models, utils
from kintsugi_tax_platform_sdk._hooks import HookContext
from kintsugi_tax_platform_sdk.types import Nullable, OptionalNullable, UNSET
from kintsugi_tax_platform_sdk.utils.unmarshal_json_response import (
    unmarshal_json_response,
)
from typing import Any, List, Mapping, Optional, Union


class Transactions(BaseSDK):
    def list(
        self,
        *,
        security: Union[
            models.GetTransactionsV1TransactionsGetSecurity,
            models.GetTransactionsV1TransactionsGetSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        state_code: OptionalNullable[str] = UNSET,
        transaction_type: OptionalNullable[str] = UNSET,
        transaction_source: OptionalNullable[str] = UNSET,
        search_query: OptionalNullable[str] = UNSET,
        country: OptionalNullable[
            Union[
                List[models.GetTransactionsV1TransactionsGetCountry],
                List[models.GetTransactionsV1TransactionsGetCountryTypedDict],
            ]
        ] = UNSET,
        state: OptionalNullable[str] = UNSET,
        address_status_in: OptionalNullable[str] = UNSET,
        status: OptionalNullable[models.TransactionStatusEnum] = UNSET,
        filing_id: OptionalNullable[str] = UNSET,
        order_by: OptionalNullable[str] = UNSET,
        date_gte: OptionalNullable[date] = UNSET,
        date_lte: OptionalNullable[date] = UNSET,
        processing_status_in: OptionalNullable[str] = UNSET,
        marketplace: OptionalNullable[bool] = UNSET,
        exempt_in: OptionalNullable[str] = UNSET,
        page: Optional[int] = 1,
        size: Optional[int] = 50,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PageTransactionRead:
        r"""Get Transactions

        The Get Transactions API retrieves a list of transactions with
        optional filtering, sorting, and pagination.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param state_code: Filter transactions by state code.
        :param transaction_type: Filter by transaction type (e.g., SALE, FULL_CREDIT_NOTE,         PARTIAL_CREDIT_NOTE, ARCHIVE etc.).
        :param transaction_source: Filter transactions based on the source.
        :param search_query: Search for transactions using a general query         (e.g., order ID, customer name).
        :param country: Filter transactions by country code         (ISO 3166-1 alpha-2 format, e.g., US).
        :param state: Filter by full state name (e.g., California).
        :param address_status_in: Filter by address status (e.g., UNVERIFIED, INVALID,         PARTIALLY_VERIFIED, VERIFIED, UNVERIFIABLE).
        :param status: Filter by transaction status (e.g., PENDING, COMMITTED,         CANCELLED, FULLY_REFUNDED, PARTIALLY_REFUNDED, ARCHIVED).
        :param filing_id: Retrieve transactions linked to a specific filing ID.
        :param order_by: Sort results based on specified fields.         Prefix with - for descending order (e.g., -date for newest first).
        :param date_gte: Retrieve transactions with a date         greater than or equal to (YYYY-MM-DD).
        :param date_lte: Retrieve transactions with a date         less than or equal to (YYYY-MM-DD).
        :param processing_status_in: Filter transactions based on processing status.         Multiple values can be passed as a comma-separated list.
        :param marketplace: Filter transactions by marketplace (e.g., AMAZON, EBAY).
        :param exempt_in: Filter transactions by exemption status.         Multiple values can be passed as a comma-separated list (e.g., EXEMPT,TAXABLE).
        :param page: Page number
        :param size: Page size
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTransactionsV1TransactionsGetRequest(
            state_code=state_code,
            transaction_type=transaction_type,
            transaction_source=transaction_source,
            search_query=search_query,
            country=country,
            state=state,
            address_status_in=address_status_in,
            status=status,
            filing_id=filing_id,
            order_by=order_by,
            date_gte=date_gte,
            date_lte=date_lte,
            processing_status_in=processing_status_in,
            marketplace=marketplace,
            exempt_in=exempt_in,
            page=page,
            size=size,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetTransactionsV1TransactionsGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transactions_v1_transactions_get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.PageTransactionRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        security: Union[
            models.GetTransactionsV1TransactionsGetSecurity,
            models.GetTransactionsV1TransactionsGetSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        state_code: OptionalNullable[str] = UNSET,
        transaction_type: OptionalNullable[str] = UNSET,
        transaction_source: OptionalNullable[str] = UNSET,
        search_query: OptionalNullable[str] = UNSET,
        country: OptionalNullable[
            Union[
                List[models.GetTransactionsV1TransactionsGetCountry],
                List[models.GetTransactionsV1TransactionsGetCountryTypedDict],
            ]
        ] = UNSET,
        state: OptionalNullable[str] = UNSET,
        address_status_in: OptionalNullable[str] = UNSET,
        status: OptionalNullable[models.TransactionStatusEnum] = UNSET,
        filing_id: OptionalNullable[str] = UNSET,
        order_by: OptionalNullable[str] = UNSET,
        date_gte: OptionalNullable[date] = UNSET,
        date_lte: OptionalNullable[date] = UNSET,
        processing_status_in: OptionalNullable[str] = UNSET,
        marketplace: OptionalNullable[bool] = UNSET,
        exempt_in: OptionalNullable[str] = UNSET,
        page: Optional[int] = 1,
        size: Optional[int] = 50,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PageTransactionRead:
        r"""Get Transactions

        The Get Transactions API retrieves a list of transactions with
        optional filtering, sorting, and pagination.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param state_code: Filter transactions by state code.
        :param transaction_type: Filter by transaction type (e.g., SALE, FULL_CREDIT_NOTE,         PARTIAL_CREDIT_NOTE, ARCHIVE etc.).
        :param transaction_source: Filter transactions based on the source.
        :param search_query: Search for transactions using a general query         (e.g., order ID, customer name).
        :param country: Filter transactions by country code         (ISO 3166-1 alpha-2 format, e.g., US).
        :param state: Filter by full state name (e.g., California).
        :param address_status_in: Filter by address status (e.g., UNVERIFIED, INVALID,         PARTIALLY_VERIFIED, VERIFIED, UNVERIFIABLE).
        :param status: Filter by transaction status (e.g., PENDING, COMMITTED,         CANCELLED, FULLY_REFUNDED, PARTIALLY_REFUNDED, ARCHIVED).
        :param filing_id: Retrieve transactions linked to a specific filing ID.
        :param order_by: Sort results based on specified fields.         Prefix with - for descending order (e.g., -date for newest first).
        :param date_gte: Retrieve transactions with a date         greater than or equal to (YYYY-MM-DD).
        :param date_lte: Retrieve transactions with a date         less than or equal to (YYYY-MM-DD).
        :param processing_status_in: Filter transactions based on processing status.         Multiple values can be passed as a comma-separated list.
        :param marketplace: Filter transactions by marketplace (e.g., AMAZON, EBAY).
        :param exempt_in: Filter transactions by exemption status.         Multiple values can be passed as a comma-separated list (e.g., EXEMPT,TAXABLE).
        :param page: Page number
        :param size: Page size
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTransactionsV1TransactionsGetRequest(
            state_code=state_code,
            transaction_type=transaction_type,
            transaction_source=transaction_source,
            search_query=search_query,
            country=country,
            state=state,
            address_status_in=address_status_in,
            status=status,
            filing_id=filing_id,
            order_by=order_by,
            date_gte=date_gte,
            date_lte=date_lte,
            processing_status_in=processing_status_in,
            marketplace=marketplace,
            exempt_in=exempt_in,
            page=page,
            size=size,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.GetTransactionsV1TransactionsGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transactions_v1_transactions_get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.PageTransactionRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        security: Union[
            models.CreateTransactionV1TransactionsPostSecurity,
            models.CreateTransactionV1TransactionsPostSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        organization_id: str,
        external_id: str,
        date_: datetime,
        addresses: Union[
            List[models.TransactionAddressPublic],
            List[models.TransactionAddressPublicTypedDict],
        ],
        transaction_items: Union[
            List[models.TransactionItemBuilder],
            List[models.TransactionItemBuilderTypedDict],
        ],
        customer: Union[models.Customer, models.CustomerTypedDict],
        type_: models.TransactionTypeEnum,
        requires_exemption: OptionalNullable[
            Union[models.ExemptionRequired, models.ExemptionRequiredTypedDict]
        ] = UNSET,
        shop_date: OptionalNullable[date] = UNSET,
        shop_date_tz: OptionalNullable[str] = UNSET,
        status: Optional[models.TransactionStatusEnum] = None,
        description: OptionalNullable[str] = UNSET,
        refund_status: OptionalNullable[models.TransactionRefundStatus] = UNSET,
        total_amount: Optional[
            Union[
                models.TransactionPublicRequestTotalAmount,
                models.TransactionPublicRequestTotalAmountTypedDict,
            ]
        ] = None,
        customer_id: OptionalNullable[str] = UNSET,
        marketplace: OptionalNullable[bool] = UNSET,
        exempt: OptionalNullable[models.TransactionExemptStatusEnum] = UNSET,
        exemptions: OptionalNullable[
            Union[List[models.Exemption], List[models.ExemptionTypedDict]]
        ] = UNSET,
        related_to: OptionalNullable[str] = UNSET,
        secondary_external_id: OptionalNullable[str] = UNSET,
        secondary_source: OptionalNullable[str] = UNSET,
        external_friendly_id: OptionalNullable[str] = UNSET,
        total_tax_amount_imported: Optional[
            Union[
                models.TransactionPublicRequestTotalTaxAmountImported,
                models.TransactionPublicRequestTotalTaxAmountImportedTypedDict,
            ]
        ] = None,
        tax_rate_imported: Optional[
            Union[
                models.TransactionPublicRequestTaxRateImported,
                models.TransactionPublicRequestTaxRateImportedTypedDict,
            ]
        ] = None,
        total_tax_amount_calculated: Optional[
            Union[
                models.TransactionPublicRequestTotalTaxAmountCalculated,
                models.TransactionPublicRequestTotalTaxAmountCalculatedTypedDict,
            ]
        ] = None,
        tax_rate_calculated: Optional[
            Union[
                models.TransactionPublicRequestTaxRateCalculated,
                models.TransactionPublicRequestTaxRateCalculatedTypedDict,
            ]
        ] = None,
        total_tax_liability_amount: Optional[
            Union[
                models.TransactionPublicRequestTotalTaxLiabilityAmount,
                models.TransactionPublicRequestTotalTaxLiabilityAmountTypedDict,
            ]
        ] = None,
        tax_liability_source: OptionalNullable[models.TaxLiabilitySourceEnum] = UNSET,
        taxable_amount: Optional[
            Union[
                models.TransactionPublicRequestTaxableAmount,
                models.TransactionPublicRequestTaxableAmountTypedDict,
            ]
        ] = None,
        currency: Optional[models.CurrencyEnum] = None,
        locked: Optional[bool] = False,
        source: Optional[models.SourceEnum] = None,
        connection_id: OptionalNullable[str] = UNSET,
        filing_id: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Create Transaction

        Create a transaction.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param organization_id: Unique identifier of the organization.
        :param external_id: External identifier of the transaction.
        :param date_: Transaction date and time
        :param addresses:
        :param transaction_items:
        :param customer:
        :param type:
        :param requires_exemption: Indicates if transaction requires tax exemption.
        :param shop_date: Transaction date in the shop's local timezone
        :param shop_date_tz: Timezone of the shop
        :param status:
        :param description: Description of the transaction.
        :param refund_status: Status of refund, if applicable
        :param total_amount: Total amount of the transaction.
        :param customer_id: Unique identifier of the customer.
        :param marketplace: Indicates if transaction is marketplace-based.
        :param exempt: Exemption status (e.g., NOT_EXEMPT)
        :param exemptions: List of exemptions applied (if any).
        :param related_to: Related transaction identifier.
        :param secondary_external_id: Secondary External Identifier.
        :param secondary_source: Secondary source information
        :param external_friendly_id: Friendly identifier of the original item.
        :param total_tax_amount_imported: Imported tax amount.
        :param tax_rate_imported: Imported tax rate.
        :param total_tax_amount_calculated: Calculated tax amount.
        :param tax_rate_calculated: Calculated tax rate.
        :param total_tax_liability_amount: Total tax liability amount.
        :param tax_liability_source: Source of tax liability.
        :param taxable_amount: Taxable amount.
        :param currency:
        :param locked: Transaction lock status.
        :param source:
        :param connection_id: Connection Identifier
        :param filing_id: Filing identifier.
        :param city: City of the transaction address.
        :param county: County of the transaction address.
        :param state: State of the transaction address.
        :param country: Country code (ISO Alpha-2).
        :param postal_code: Postal code of the transaction.
        :param tax_id: Tax ID associated with the transaction
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateTransactionV1TransactionsPostRequest(
            x_organization_id=x_organization_id,
            transaction_public_request=models.TransactionPublicRequest(
                requires_exemption=utils.get_pydantic_model(
                    requires_exemption, OptionalNullable[models.ExemptionRequired]
                ),
                organization_id=organization_id,
                external_id=external_id,
                date_=date_,
                shop_date=shop_date,
                shop_date_tz=shop_date_tz,
                status=status,
                description=description,
                refund_status=refund_status,
                total_amount=total_amount,
                customer_id=customer_id,
                marketplace=marketplace,
                exempt=exempt,
                exemptions=utils.get_pydantic_model(
                    exemptions, OptionalNullable[List[models.Exemption]]
                ),
                related_to=related_to,
                secondary_external_id=secondary_external_id,
                secondary_source=secondary_source,
                external_friendly_id=external_friendly_id,
                total_tax_amount_imported=total_tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                total_tax_amount_calculated=total_tax_amount_calculated,
                tax_rate_calculated=tax_rate_calculated,
                total_tax_liability_amount=total_tax_liability_amount,
                tax_liability_source=tax_liability_source,
                taxable_amount=taxable_amount,
                currency=currency,
                locked=locked,
                source=source,
                connection_id=connection_id,
                filing_id=filing_id,
                city=city,
                county=county,
                state=state,
                country=country,
                postal_code=postal_code,
                tax_id=tax_id,
                addresses=utils.get_pydantic_model(
                    addresses, List[models.TransactionAddressPublic]
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.TransactionItemBuilder]
                ),
                customer=utils.get_pydantic_model(customer, models.Customer),
                type=type_,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CreateTransactionV1TransactionsPostSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.transaction_public_request,
                False,
                False,
                "json",
                models.TransactionPublicRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_transaction_v1_transactions_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "202", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, ["400", "401"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        security: Union[
            models.CreateTransactionV1TransactionsPostSecurity,
            models.CreateTransactionV1TransactionsPostSecurityTypedDict,
        ],
        x_organization_id: Nullable[str],
        organization_id: str,
        external_id: str,
        date_: datetime,
        addresses: Union[
            List[models.TransactionAddressPublic],
            List[models.TransactionAddressPublicTypedDict],
        ],
        transaction_items: Union[
            List[models.TransactionItemBuilder],
            List[models.TransactionItemBuilderTypedDict],
        ],
        customer: Union[models.Customer, models.CustomerTypedDict],
        type_: models.TransactionTypeEnum,
        requires_exemption: OptionalNullable[
            Union[models.ExemptionRequired, models.ExemptionRequiredTypedDict]
        ] = UNSET,
        shop_date: OptionalNullable[date] = UNSET,
        shop_date_tz: OptionalNullable[str] = UNSET,
        status: Optional[models.TransactionStatusEnum] = None,
        description: OptionalNullable[str] = UNSET,
        refund_status: OptionalNullable[models.TransactionRefundStatus] = UNSET,
        total_amount: Optional[
            Union[
                models.TransactionPublicRequestTotalAmount,
                models.TransactionPublicRequestTotalAmountTypedDict,
            ]
        ] = None,
        customer_id: OptionalNullable[str] = UNSET,
        marketplace: OptionalNullable[bool] = UNSET,
        exempt: OptionalNullable[models.TransactionExemptStatusEnum] = UNSET,
        exemptions: OptionalNullable[
            Union[List[models.Exemption], List[models.ExemptionTypedDict]]
        ] = UNSET,
        related_to: OptionalNullable[str] = UNSET,
        secondary_external_id: OptionalNullable[str] = UNSET,
        secondary_source: OptionalNullable[str] = UNSET,
        external_friendly_id: OptionalNullable[str] = UNSET,
        total_tax_amount_imported: Optional[
            Union[
                models.TransactionPublicRequestTotalTaxAmountImported,
                models.TransactionPublicRequestTotalTaxAmountImportedTypedDict,
            ]
        ] = None,
        tax_rate_imported: Optional[
            Union[
                models.TransactionPublicRequestTaxRateImported,
                models.TransactionPublicRequestTaxRateImportedTypedDict,
            ]
        ] = None,
        total_tax_amount_calculated: Optional[
            Union[
                models.TransactionPublicRequestTotalTaxAmountCalculated,
                models.TransactionPublicRequestTotalTaxAmountCalculatedTypedDict,
            ]
        ] = None,
        tax_rate_calculated: Optional[
            Union[
                models.TransactionPublicRequestTaxRateCalculated,
                models.TransactionPublicRequestTaxRateCalculatedTypedDict,
            ]
        ] = None,
        total_tax_liability_amount: Optional[
            Union[
                models.TransactionPublicRequestTotalTaxLiabilityAmount,
                models.TransactionPublicRequestTotalTaxLiabilityAmountTypedDict,
            ]
        ] = None,
        tax_liability_source: OptionalNullable[models.TaxLiabilitySourceEnum] = UNSET,
        taxable_amount: Optional[
            Union[
                models.TransactionPublicRequestTaxableAmount,
                models.TransactionPublicRequestTaxableAmountTypedDict,
            ]
        ] = None,
        currency: Optional[models.CurrencyEnum] = None,
        locked: Optional[bool] = False,
        source: Optional[models.SourceEnum] = None,
        connection_id: OptionalNullable[str] = UNSET,
        filing_id: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Create Transaction

        Create a transaction.

        :param security:
        :param x_organization_id: The unique identifier for the organization making the request
        :param organization_id: Unique identifier of the organization.
        :param external_id: External identifier of the transaction.
        :param date_: Transaction date and time
        :param addresses:
        :param transaction_items:
        :param customer:
        :param type:
        :param requires_exemption: Indicates if transaction requires tax exemption.
        :param shop_date: Transaction date in the shop's local timezone
        :param shop_date_tz: Timezone of the shop
        :param status:
        :param description: Description of the transaction.
        :param refund_status: Status of refund, if applicable
        :param total_amount: Total amount of the transaction.
        :param customer_id: Unique identifier of the customer.
        :param marketplace: Indicates if transaction is marketplace-based.
        :param exempt: Exemption status (e.g., NOT_EXEMPT)
        :param exemptions: List of exemptions applied (if any).
        :param related_to: Related transaction identifier.
        :param secondary_external_id: Secondary External Identifier.
        :param secondary_source: Secondary source information
        :param external_friendly_id: Friendly identifier of the original item.
        :param total_tax_amount_imported: Imported tax amount.
        :param tax_rate_imported: Imported tax rate.
        :param total_tax_amount_calculated: Calculated tax amount.
        :param tax_rate_calculated: Calculated tax rate.
        :param total_tax_liability_amount: Total tax liability amount.
        :param tax_liability_source: Source of tax liability.
        :param taxable_amount: Taxable amount.
        :param currency:
        :param locked: Transaction lock status.
        :param source:
        :param connection_id: Connection Identifier
        :param filing_id: Filing identifier.
        :param city: City of the transaction address.
        :param county: County of the transaction address.
        :param state: State of the transaction address.
        :param country: Country code (ISO Alpha-2).
        :param postal_code: Postal code of the transaction.
        :param tax_id: Tax ID associated with the transaction
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateTransactionV1TransactionsPostRequest(
            x_organization_id=x_organization_id,
            transaction_public_request=models.TransactionPublicRequest(
                requires_exemption=utils.get_pydantic_model(
                    requires_exemption, OptionalNullable[models.ExemptionRequired]
                ),
                organization_id=organization_id,
                external_id=external_id,
                date_=date_,
                shop_date=shop_date,
                shop_date_tz=shop_date_tz,
                status=status,
                description=description,
                refund_status=refund_status,
                total_amount=total_amount,
                customer_id=customer_id,
                marketplace=marketplace,
                exempt=exempt,
                exemptions=utils.get_pydantic_model(
                    exemptions, OptionalNullable[List[models.Exemption]]
                ),
                related_to=related_to,
                secondary_external_id=secondary_external_id,
                secondary_source=secondary_source,
                external_friendly_id=external_friendly_id,
                total_tax_amount_imported=total_tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                total_tax_amount_calculated=total_tax_amount_calculated,
                tax_rate_calculated=tax_rate_calculated,
                total_tax_liability_amount=total_tax_liability_amount,
                tax_liability_source=tax_liability_source,
                taxable_amount=taxable_amount,
                currency=currency,
                locked=locked,
                source=source,
                connection_id=connection_id,
                filing_id=filing_id,
                city=city,
                county=county,
                state=state,
                country=country,
                postal_code=postal_code,
                tax_id=tax_id,
                addresses=utils.get_pydantic_model(
                    addresses, List[models.TransactionAddressPublic]
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.TransactionItemBuilder]
                ),
                customer=utils.get_pydantic_model(customer, models.Customer),
                type=type_,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CreateTransactionV1TransactionsPostSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.transaction_public_request,
                False,
                False,
                "json",
                models.TransactionPublicRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_transaction_v1_transactions_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "202", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, ["400", "401"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_by_external_id(
        self,
        *,
        security: Union[
            models.GetTransactionByExternalIDV1TransactionsExternalExternalIDGetSecurity,
            models.GetTransactionByExternalIDV1TransactionsExternalExternalIDGetSecurityTypedDict,
        ],
        external_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Get Transaction By External Id

        Retrieves a specific transaction based on its external ID.
        This allows users to fetch transaction details using an identifier from an external system.

        :param security:
        :param external_id: The unique external identifier of the transaction.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.GetTransactionByExternalIDV1TransactionsExternalExternalIDGetRequest(
                external_id=external_id,
                x_organization_id=x_organization_id,
            )
        )

        req = self._build_request(
            method="GET",
            path="/v1/transactions/external/{external_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetTransactionByExternalIDV1TransactionsExternalExternalIDGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transaction_by_external_id_v1_transactions_external__external_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_by_external_id_async(
        self,
        *,
        security: Union[
            models.GetTransactionByExternalIDV1TransactionsExternalExternalIDGetSecurity,
            models.GetTransactionByExternalIDV1TransactionsExternalExternalIDGetSecurityTypedDict,
        ],
        external_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Get Transaction By External Id

        Retrieves a specific transaction based on its external ID.
        This allows users to fetch transaction details using an identifier from an external system.

        :param security:
        :param external_id: The unique external identifier of the transaction.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.GetTransactionByExternalIDV1TransactionsExternalExternalIDGetRequest(
                external_id=external_id,
                x_organization_id=x_organization_id,
            )
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/transactions/external/{external_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetTransactionByExternalIDV1TransactionsExternalExternalIDGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transaction_by_external_id_v1_transactions_external__external_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        security: Union[
            models.UpdateTransactionV1TransactionsTransactionIDPutSecurity,
            models.UpdateTransactionV1TransactionsTransactionIDPutSecurityTypedDict,
        ],
        transaction_id: str,
        x_organization_id: Nullable[str],
        organization_id: str,
        external_id: str,
        date_: datetime,
        addresses: Union[
            models.TransactionUpdateAddresses,
            models.TransactionUpdateAddressesTypedDict,
        ],
        transaction_items: Union[
            List[models.TransactionItemCreateUpdate],
            List[models.TransactionItemCreateUpdateTypedDict],
        ],
        customer: Union[models.CustomerUpdate, models.CustomerUpdateTypedDict],
        requires_exemption: OptionalNullable[
            Union[models.ExemptionRequired, models.ExemptionRequiredTypedDict]
        ] = UNSET,
        shop_date: OptionalNullable[date] = UNSET,
        shop_date_tz: OptionalNullable[str] = UNSET,
        status: Optional[models.TransactionStatusEnum] = None,
        description: OptionalNullable[str] = UNSET,
        refund_status: OptionalNullable[models.TransactionRefundStatus] = UNSET,
        total_amount: Optional[
            Union[
                models.TransactionUpdateTotalAmount,
                models.TransactionUpdateTotalAmountTypedDict,
            ]
        ] = None,
        customer_id: OptionalNullable[str] = UNSET,
        marketplace: OptionalNullable[bool] = UNSET,
        exempt: OptionalNullable[models.TransactionExemptStatusEnum] = UNSET,
        exemptions: OptionalNullable[
            Union[List[models.Exemption], List[models.ExemptionTypedDict]]
        ] = UNSET,
        related_to: OptionalNullable[str] = UNSET,
        secondary_external_id: OptionalNullable[str] = UNSET,
        secondary_source: OptionalNullable[str] = UNSET,
        external_friendly_id: OptionalNullable[str] = UNSET,
        total_tax_amount_imported: Optional[
            Union[
                models.TransactionUpdateTotalTaxAmountImported,
                models.TransactionUpdateTotalTaxAmountImportedTypedDict,
            ]
        ] = None,
        tax_rate_imported: Optional[
            Union[
                models.TransactionUpdateTaxRateImported,
                models.TransactionUpdateTaxRateImportedTypedDict,
            ]
        ] = None,
        total_tax_amount_calculated: Optional[
            Union[
                models.TransactionUpdateTotalTaxAmountCalculated,
                models.TransactionUpdateTotalTaxAmountCalculatedTypedDict,
            ]
        ] = None,
        tax_rate_calculated: Optional[
            Union[
                models.TransactionUpdateTaxRateCalculated,
                models.TransactionUpdateTaxRateCalculatedTypedDict,
            ]
        ] = None,
        total_tax_liability_amount: Optional[
            Union[
                models.TransactionUpdateTotalTaxLiabilityAmount,
                models.TransactionUpdateTotalTaxLiabilityAmountTypedDict,
            ]
        ] = None,
        tax_liability_source: OptionalNullable[models.TaxLiabilitySourceEnum] = UNSET,
        taxable_amount: Optional[
            Union[
                models.TransactionUpdateTaxableAmount,
                models.TransactionUpdateTaxableAmountTypedDict,
            ]
        ] = None,
        currency: Optional[models.CurrencyEnum] = None,
        locked: Optional[bool] = False,
        source: Optional[models.SourceEnum] = None,
        connection_id: OptionalNullable[str] = UNSET,
        filing_id: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        address_status: Optional[models.AddressStatus] = None,
        processing_status: Optional[models.ProcessingStatusEnum] = None,
        destination_currency: OptionalNullable[models.CurrencyEnum] = UNSET,
        converted_total_amount: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalAmount,
                models.TransactionUpdateConvertedTotalAmountTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_amount_imported: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalTaxAmountImported,
                models.TransactionUpdateConvertedTotalTaxAmountImportedTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_amount_calculated: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalTaxAmountCalculated,
                models.TransactionUpdateConvertedTotalTaxAmountCalculatedTypedDict,
            ]
        ] = UNSET,
        conversion_rate: OptionalNullable[
            Union[
                models.TransactionUpdateConversionRate,
                models.TransactionUpdateConversionRateTypedDict,
            ]
        ] = UNSET,
        converted_taxable_amount: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTaxableAmount,
                models.TransactionUpdateConvertedTaxableAmountTypedDict,
            ]
        ] = UNSET,
        converted_total_discount: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalDiscount,
                models.TransactionUpdateConvertedTotalDiscountTypedDict,
            ]
        ] = UNSET,
        converted_subtotal: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedSubtotal,
                models.TransactionUpdateConvertedSubtotalTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_liability_amount: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalTaxLiabilityAmount,
                models.TransactionUpdateConvertedTotalTaxLiabilityAmountTypedDict,
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Update Transaction

        Update a specific transaction by its ID.

        :param security:
        :param transaction_id:
        :param x_organization_id: The unique identifier for the organization making the request
        :param organization_id: Unique identifier of the organization.
        :param external_id: External identifier of the transaction.
        :param date_: Transaction date and time
        :param addresses:
        :param transaction_items:
        :param customer:
        :param requires_exemption: Indicates if transaction requires tax exemption.
        :param shop_date: Transaction date in the shop's local timezone
        :param shop_date_tz: Timezone of the shop
        :param status:
        :param description: Description of the transaction.
        :param refund_status: Status of refund, if applicable
        :param total_amount: Total amount of the transaction.
        :param customer_id: Unique identifier of the customer.
        :param marketplace: Indicates if transaction is marketplace-based.
        :param exempt: Exemption status (e.g., NOT_EXEMPT)
        :param exemptions: List of exemptions applied (if any).
        :param related_to: Related transaction identifier.
        :param secondary_external_id: Secondary External Identifier.
        :param secondary_source: Secondary source information
        :param external_friendly_id: Friendly identifier of the original item.
        :param total_tax_amount_imported: Imported tax amount.
        :param tax_rate_imported: Imported tax rate.
        :param total_tax_amount_calculated: Calculated tax amount.
        :param tax_rate_calculated: Calculated tax rate.
        :param total_tax_liability_amount: Total tax liability amount.
        :param tax_liability_source: Source of tax liability.
        :param taxable_amount: Taxable amount.
        :param currency:
        :param locked: Transaction lock status.
        :param source:
        :param connection_id: Connection Identifier
        :param filing_id: Filing identifier.
        :param city: City of the transaction address.
        :param county: County of the transaction address.
        :param state: State of the transaction address.
        :param country: Country code (ISO Alpha-2).
        :param postal_code: Postal code of the transaction.
        :param tax_id: Tax ID associated with the transaction
        :param address_status:
        :param processing_status: Our transaction state, used to determine when/if a transaction needs additional processing.
        :param destination_currency: Destination currency code (ISO 4217, e.g., USD)
        :param converted_total_amount: Converted total amount.
        :param converted_total_tax_amount_imported: Converted imported tax amount.
        :param converted_total_tax_amount_calculated: Converted calculated tax amount.
        :param conversion_rate: Currency conversion rate.
        :param converted_taxable_amount: Converted taxable amount.
        :param converted_total_discount: Converted total discount amount.
        :param converted_subtotal: Converted subtotal amount.
        :param converted_total_tax_liability_amount: Converted total tax liability amount.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateTransactionV1TransactionsTransactionIDPutRequest(
            transaction_id=transaction_id,
            x_organization_id=x_organization_id,
            transaction_update=models.TransactionUpdate(
                requires_exemption=utils.get_pydantic_model(
                    requires_exemption, OptionalNullable[models.ExemptionRequired]
                ),
                organization_id=organization_id,
                external_id=external_id,
                date_=date_,
                shop_date=shop_date,
                shop_date_tz=shop_date_tz,
                status=status,
                description=description,
                refund_status=refund_status,
                total_amount=total_amount,
                customer_id=customer_id,
                marketplace=marketplace,
                exempt=exempt,
                exemptions=utils.get_pydantic_model(
                    exemptions, OptionalNullable[List[models.Exemption]]
                ),
                related_to=related_to,
                secondary_external_id=secondary_external_id,
                secondary_source=secondary_source,
                external_friendly_id=external_friendly_id,
                total_tax_amount_imported=total_tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                total_tax_amount_calculated=total_tax_amount_calculated,
                tax_rate_calculated=tax_rate_calculated,
                total_tax_liability_amount=total_tax_liability_amount,
                tax_liability_source=tax_liability_source,
                taxable_amount=taxable_amount,
                currency=currency,
                locked=locked,
                source=source,
                connection_id=connection_id,
                filing_id=filing_id,
                city=city,
                county=county,
                state=state,
                country=country,
                postal_code=postal_code,
                tax_id=tax_id,
                address_status=address_status,
                processing_status=processing_status,
                destination_currency=destination_currency,
                converted_total_amount=converted_total_amount,
                converted_total_tax_amount_imported=converted_total_tax_amount_imported,
                converted_total_tax_amount_calculated=converted_total_tax_amount_calculated,
                conversion_rate=conversion_rate,
                converted_taxable_amount=converted_taxable_amount,
                converted_total_discount=converted_total_discount,
                converted_subtotal=converted_subtotal,
                converted_total_tax_liability_amount=converted_total_tax_liability_amount,
                addresses=utils.get_pydantic_model(
                    addresses, models.TransactionUpdateAddresses
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.TransactionItemCreateUpdate]
                ),
                customer=utils.get_pydantic_model(customer, models.CustomerUpdate),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/transactions/{transaction_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.UpdateTransactionV1TransactionsTransactionIDPutSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.transaction_update,
                False,
                False,
                "json",
                models.TransactionUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_transaction_v1_transactions__transaction_id__put",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        security: Union[
            models.UpdateTransactionV1TransactionsTransactionIDPutSecurity,
            models.UpdateTransactionV1TransactionsTransactionIDPutSecurityTypedDict,
        ],
        transaction_id: str,
        x_organization_id: Nullable[str],
        organization_id: str,
        external_id: str,
        date_: datetime,
        addresses: Union[
            models.TransactionUpdateAddresses,
            models.TransactionUpdateAddressesTypedDict,
        ],
        transaction_items: Union[
            List[models.TransactionItemCreateUpdate],
            List[models.TransactionItemCreateUpdateTypedDict],
        ],
        customer: Union[models.CustomerUpdate, models.CustomerUpdateTypedDict],
        requires_exemption: OptionalNullable[
            Union[models.ExemptionRequired, models.ExemptionRequiredTypedDict]
        ] = UNSET,
        shop_date: OptionalNullable[date] = UNSET,
        shop_date_tz: OptionalNullable[str] = UNSET,
        status: Optional[models.TransactionStatusEnum] = None,
        description: OptionalNullable[str] = UNSET,
        refund_status: OptionalNullable[models.TransactionRefundStatus] = UNSET,
        total_amount: Optional[
            Union[
                models.TransactionUpdateTotalAmount,
                models.TransactionUpdateTotalAmountTypedDict,
            ]
        ] = None,
        customer_id: OptionalNullable[str] = UNSET,
        marketplace: OptionalNullable[bool] = UNSET,
        exempt: OptionalNullable[models.TransactionExemptStatusEnum] = UNSET,
        exemptions: OptionalNullable[
            Union[List[models.Exemption], List[models.ExemptionTypedDict]]
        ] = UNSET,
        related_to: OptionalNullable[str] = UNSET,
        secondary_external_id: OptionalNullable[str] = UNSET,
        secondary_source: OptionalNullable[str] = UNSET,
        external_friendly_id: OptionalNullable[str] = UNSET,
        total_tax_amount_imported: Optional[
            Union[
                models.TransactionUpdateTotalTaxAmountImported,
                models.TransactionUpdateTotalTaxAmountImportedTypedDict,
            ]
        ] = None,
        tax_rate_imported: Optional[
            Union[
                models.TransactionUpdateTaxRateImported,
                models.TransactionUpdateTaxRateImportedTypedDict,
            ]
        ] = None,
        total_tax_amount_calculated: Optional[
            Union[
                models.TransactionUpdateTotalTaxAmountCalculated,
                models.TransactionUpdateTotalTaxAmountCalculatedTypedDict,
            ]
        ] = None,
        tax_rate_calculated: Optional[
            Union[
                models.TransactionUpdateTaxRateCalculated,
                models.TransactionUpdateTaxRateCalculatedTypedDict,
            ]
        ] = None,
        total_tax_liability_amount: Optional[
            Union[
                models.TransactionUpdateTotalTaxLiabilityAmount,
                models.TransactionUpdateTotalTaxLiabilityAmountTypedDict,
            ]
        ] = None,
        tax_liability_source: OptionalNullable[models.TaxLiabilitySourceEnum] = UNSET,
        taxable_amount: Optional[
            Union[
                models.TransactionUpdateTaxableAmount,
                models.TransactionUpdateTaxableAmountTypedDict,
            ]
        ] = None,
        currency: Optional[models.CurrencyEnum] = None,
        locked: Optional[bool] = False,
        source: Optional[models.SourceEnum] = None,
        connection_id: OptionalNullable[str] = UNSET,
        filing_id: OptionalNullable[str] = UNSET,
        city: OptionalNullable[str] = UNSET,
        county: OptionalNullable[str] = UNSET,
        state: OptionalNullable[str] = UNSET,
        country: OptionalNullable[models.CountryCodeEnum] = UNSET,
        postal_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        address_status: Optional[models.AddressStatus] = None,
        processing_status: Optional[models.ProcessingStatusEnum] = None,
        destination_currency: OptionalNullable[models.CurrencyEnum] = UNSET,
        converted_total_amount: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalAmount,
                models.TransactionUpdateConvertedTotalAmountTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_amount_imported: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalTaxAmountImported,
                models.TransactionUpdateConvertedTotalTaxAmountImportedTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_amount_calculated: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalTaxAmountCalculated,
                models.TransactionUpdateConvertedTotalTaxAmountCalculatedTypedDict,
            ]
        ] = UNSET,
        conversion_rate: OptionalNullable[
            Union[
                models.TransactionUpdateConversionRate,
                models.TransactionUpdateConversionRateTypedDict,
            ]
        ] = UNSET,
        converted_taxable_amount: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTaxableAmount,
                models.TransactionUpdateConvertedTaxableAmountTypedDict,
            ]
        ] = UNSET,
        converted_total_discount: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalDiscount,
                models.TransactionUpdateConvertedTotalDiscountTypedDict,
            ]
        ] = UNSET,
        converted_subtotal: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedSubtotal,
                models.TransactionUpdateConvertedSubtotalTypedDict,
            ]
        ] = UNSET,
        converted_total_tax_liability_amount: OptionalNullable[
            Union[
                models.TransactionUpdateConvertedTotalTaxLiabilityAmount,
                models.TransactionUpdateConvertedTotalTaxLiabilityAmountTypedDict,
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Update Transaction

        Update a specific transaction by its ID.

        :param security:
        :param transaction_id:
        :param x_organization_id: The unique identifier for the organization making the request
        :param organization_id: Unique identifier of the organization.
        :param external_id: External identifier of the transaction.
        :param date_: Transaction date and time
        :param addresses:
        :param transaction_items:
        :param customer:
        :param requires_exemption: Indicates if transaction requires tax exemption.
        :param shop_date: Transaction date in the shop's local timezone
        :param shop_date_tz: Timezone of the shop
        :param status:
        :param description: Description of the transaction.
        :param refund_status: Status of refund, if applicable
        :param total_amount: Total amount of the transaction.
        :param customer_id: Unique identifier of the customer.
        :param marketplace: Indicates if transaction is marketplace-based.
        :param exempt: Exemption status (e.g., NOT_EXEMPT)
        :param exemptions: List of exemptions applied (if any).
        :param related_to: Related transaction identifier.
        :param secondary_external_id: Secondary External Identifier.
        :param secondary_source: Secondary source information
        :param external_friendly_id: Friendly identifier of the original item.
        :param total_tax_amount_imported: Imported tax amount.
        :param tax_rate_imported: Imported tax rate.
        :param total_tax_amount_calculated: Calculated tax amount.
        :param tax_rate_calculated: Calculated tax rate.
        :param total_tax_liability_amount: Total tax liability amount.
        :param tax_liability_source: Source of tax liability.
        :param taxable_amount: Taxable amount.
        :param currency:
        :param locked: Transaction lock status.
        :param source:
        :param connection_id: Connection Identifier
        :param filing_id: Filing identifier.
        :param city: City of the transaction address.
        :param county: County of the transaction address.
        :param state: State of the transaction address.
        :param country: Country code (ISO Alpha-2).
        :param postal_code: Postal code of the transaction.
        :param tax_id: Tax ID associated with the transaction
        :param address_status:
        :param processing_status: Our transaction state, used to determine when/if a transaction needs additional processing.
        :param destination_currency: Destination currency code (ISO 4217, e.g., USD)
        :param converted_total_amount: Converted total amount.
        :param converted_total_tax_amount_imported: Converted imported tax amount.
        :param converted_total_tax_amount_calculated: Converted calculated tax amount.
        :param conversion_rate: Currency conversion rate.
        :param converted_taxable_amount: Converted taxable amount.
        :param converted_total_discount: Converted total discount amount.
        :param converted_subtotal: Converted subtotal amount.
        :param converted_total_tax_liability_amount: Converted total tax liability amount.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateTransactionV1TransactionsTransactionIDPutRequest(
            transaction_id=transaction_id,
            x_organization_id=x_organization_id,
            transaction_update=models.TransactionUpdate(
                requires_exemption=utils.get_pydantic_model(
                    requires_exemption, OptionalNullable[models.ExemptionRequired]
                ),
                organization_id=organization_id,
                external_id=external_id,
                date_=date_,
                shop_date=shop_date,
                shop_date_tz=shop_date_tz,
                status=status,
                description=description,
                refund_status=refund_status,
                total_amount=total_amount,
                customer_id=customer_id,
                marketplace=marketplace,
                exempt=exempt,
                exemptions=utils.get_pydantic_model(
                    exemptions, OptionalNullable[List[models.Exemption]]
                ),
                related_to=related_to,
                secondary_external_id=secondary_external_id,
                secondary_source=secondary_source,
                external_friendly_id=external_friendly_id,
                total_tax_amount_imported=total_tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                total_tax_amount_calculated=total_tax_amount_calculated,
                tax_rate_calculated=tax_rate_calculated,
                total_tax_liability_amount=total_tax_liability_amount,
                tax_liability_source=tax_liability_source,
                taxable_amount=taxable_amount,
                currency=currency,
                locked=locked,
                source=source,
                connection_id=connection_id,
                filing_id=filing_id,
                city=city,
                county=county,
                state=state,
                country=country,
                postal_code=postal_code,
                tax_id=tax_id,
                address_status=address_status,
                processing_status=processing_status,
                destination_currency=destination_currency,
                converted_total_amount=converted_total_amount,
                converted_total_tax_amount_imported=converted_total_tax_amount_imported,
                converted_total_tax_amount_calculated=converted_total_tax_amount_calculated,
                conversion_rate=conversion_rate,
                converted_taxable_amount=converted_taxable_amount,
                converted_total_discount=converted_total_discount,
                converted_subtotal=converted_subtotal,
                converted_total_tax_liability_amount=converted_total_tax_liability_amount,
                addresses=utils.get_pydantic_model(
                    addresses, models.TransactionUpdateAddresses
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.TransactionItemCreateUpdate]
                ),
                customer=utils.get_pydantic_model(customer, models.CustomerUpdate),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/transactions/{transaction_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.UpdateTransactionV1TransactionsTransactionIDPutSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.transaction_update,
                False,
                False,
                "json",
                models.TransactionUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_transaction_v1_transactions__transaction_id__put",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_by_id(
        self,
        *,
        security: Union[
            models.GetTransactionByIDV1TransactionsTransactionIDGetSecurity,
            models.GetTransactionByIDV1TransactionsTransactionIDGetSecurityTypedDict,
        ],
        transaction_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Get Transaction By Id

        The Get Transaction By Id API retrieves detailed information
        about a specific transaction by providing its unique transaction ID.

        :param security:
        :param transaction_id: The unique identifier of the transaction to retrieve.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTransactionByIDV1TransactionsTransactionIDGetRequest(
            transaction_id=transaction_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/transactions/{transaction_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetTransactionByIDV1TransactionsTransactionIDGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transaction_by_id_v1_transactions__transaction_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_by_id_async(
        self,
        *,
        security: Union[
            models.GetTransactionByIDV1TransactionsTransactionIDGetSecurity,
            models.GetTransactionByIDV1TransactionsTransactionIDGetSecurityTypedDict,
        ],
        transaction_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Get Transaction By Id

        The Get Transaction By Id API retrieves detailed information
        about a specific transaction by providing its unique transaction ID.

        :param security:
        :param transaction_id: The unique identifier of the transaction to retrieve.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTransactionByIDV1TransactionsTransactionIDGetRequest(
            transaction_id=transaction_id,
            x_organization_id=x_organization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/transactions/{transaction_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetTransactionByIDV1TransactionsTransactionIDGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transaction_by_id_v1_transactions__transaction_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "404", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, ["401", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_by_filing_id(
        self,
        *,
        security: Union[
            models.GetTransactionsByFilingIDV1TransactionsFilingsFilingIDGetSecurity,
            models.GetTransactionsByFilingIDV1TransactionsFilingsFilingIDGetSecurityTypedDict,
        ],
        filing_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.TransactionRead]:
        r"""Get Transactions By Filing Id

        Retrieve transactions by filing ID.

        :param security:
        :param filing_id: The unique identifier of the filing         whose transactions you wish to retrieve.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.GetTransactionsByFilingIDV1TransactionsFilingsFilingIDGetRequest(
                filing_id=filing_id,
                x_organization_id=x_organization_id,
            )
        )

        req = self._build_request(
            method="GET",
            path="/v1/transactions/filings/{filing_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetTransactionsByFilingIDV1TransactionsFilingsFilingIDGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transactions_by_filing_id_v1_transactions_filings__filing_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.TransactionRead], http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_by_filing_id_async(
        self,
        *,
        security: Union[
            models.GetTransactionsByFilingIDV1TransactionsFilingsFilingIDGetSecurity,
            models.GetTransactionsByFilingIDV1TransactionsFilingsFilingIDGetSecurityTypedDict,
        ],
        filing_id: str,
        x_organization_id: Nullable[str],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.TransactionRead]:
        r"""Get Transactions By Filing Id

        Retrieve transactions by filing ID.

        :param security:
        :param filing_id: The unique identifier of the filing         whose transactions you wish to retrieve.
        :param x_organization_id: The unique identifier for the organization making the request
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.GetTransactionsByFilingIDV1TransactionsFilingsFilingIDGetRequest(
                filing_id=filing_id,
                x_organization_id=x_organization_id,
            )
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/transactions/filings/{filing_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.GetTransactionsByFilingIDV1TransactionsFilingsFilingIDGetSecurity,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_transactions_by_filing_id_v1_transactions_filings__filing_id__get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["401", "422", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.TransactionRead], http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BackendSrcTransactionsResponsesValidationErrorResponseData,
                http_res,
            )
            raise errors.BackendSrcTransactionsResponsesValidationErrorResponse(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def create_credit_note(
        self,
        *,
        security: Union[
            models.CreateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesPostSecurity,
            models.CreateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesPostSecurityTypedDict,
        ],
        original_transaction_id: str,
        x_organization_id: Nullable[str],
        external_id: str,
        date_: datetime,
        status: models.Status,
        currency: models.CurrencyEnum,
        transaction_items: Union[
            List[models.CreditNoteItemCreateUpdate],
            List[models.CreditNoteItemCreateUpdateTypedDict],
        ],
        description: OptionalNullable[str] = UNSET,
        total_amount: Optional[
            Union[
                models.CreditNoteCreateTotalAmount,
                models.CreditNoteCreateTotalAmountTypedDict,
            ]
        ] = None,
        marketplace: OptionalNullable[bool] = UNSET,
        tax_amount_imported: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxAmountImported,
                models.CreditNoteCreateTaxAmountImportedTypedDict,
            ]
        ] = UNSET,
        tax_rate_imported: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxRateImported,
                models.CreditNoteCreateTaxRateImportedTypedDict,
            ]
        ] = UNSET,
        taxable_amount: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxableAmount,
                models.CreditNoteCreateTaxableAmountTypedDict,
            ]
        ] = UNSET,
        addresses: OptionalNullable[
            Union[
                List[models.TransactionAddressBuilder],
                List[models.TransactionAddressBuilderTypedDict],
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Create Credit Note By Transaction Id

        Create a new credit note for a specific transaction.

        :param security:
        :param original_transaction_id:
        :param x_organization_id: The unique identifier for the organization making the request
        :param external_id: Unique identifier for the credit note in the external system.
        :param date_: Date when the credit note was issued or created.
        :param status: Current state of the credit note in its lifecycle.
        :param currency:
        :param transaction_items: Detailed list of individual items included in this credit note.
        :param description: Brief explanation or reason for issuing the credit note.
        :param total_amount: Total monetary value of the credit note, including all items and taxes.
        :param marketplace: Indicates whether this credit note is associated with a marketplace transaction.
        :param tax_amount_imported: Pre-calculated total tax amount for the entire credit note, if provided by the external system.
        :param tax_rate_imported: Pre-calculated overall tax rate for the credit note, if provided by the external system.
        :param taxable_amount: Total portion of the credit note amount subject to taxation.
        :param addresses: A list of TransactionAddressBuilder objects or None if no addresses are provided. This field represents the addresses associated with the transaction.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesPostRequest(
            original_transaction_id=original_transaction_id,
            x_organization_id=x_organization_id,
            credit_note_create=models.CreditNoteCreate(
                external_id=external_id,
                date_=date_,
                status=status,
                description=description,
                total_amount=total_amount,
                marketplace=marketplace,
                tax_amount_imported=tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                taxable_amount=taxable_amount,
                currency=currency,
                addresses=utils.get_pydantic_model(
                    addresses, OptionalNullable[List[models.TransactionAddressBuilder]]
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.CreditNoteItemCreateUpdate]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/transactions/{original_transaction_id}/credit_notes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.CreateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesPostSecurity,
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.credit_note_create,
                False,
                False,
                "json",
                models.CreditNoteCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_credit_note_by_transaction_id_v1_transactions__original_transaction_id__credit_notes_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_credit_note_async(
        self,
        *,
        security: Union[
            models.CreateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesPostSecurity,
            models.CreateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesPostSecurityTypedDict,
        ],
        original_transaction_id: str,
        x_organization_id: Nullable[str],
        external_id: str,
        date_: datetime,
        status: models.Status,
        currency: models.CurrencyEnum,
        transaction_items: Union[
            List[models.CreditNoteItemCreateUpdate],
            List[models.CreditNoteItemCreateUpdateTypedDict],
        ],
        description: OptionalNullable[str] = UNSET,
        total_amount: Optional[
            Union[
                models.CreditNoteCreateTotalAmount,
                models.CreditNoteCreateTotalAmountTypedDict,
            ]
        ] = None,
        marketplace: OptionalNullable[bool] = UNSET,
        tax_amount_imported: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxAmountImported,
                models.CreditNoteCreateTaxAmountImportedTypedDict,
            ]
        ] = UNSET,
        tax_rate_imported: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxRateImported,
                models.CreditNoteCreateTaxRateImportedTypedDict,
            ]
        ] = UNSET,
        taxable_amount: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxableAmount,
                models.CreditNoteCreateTaxableAmountTypedDict,
            ]
        ] = UNSET,
        addresses: OptionalNullable[
            Union[
                List[models.TransactionAddressBuilder],
                List[models.TransactionAddressBuilderTypedDict],
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransactionRead:
        r"""Create Credit Note By Transaction Id

        Create a new credit note for a specific transaction.

        :param security:
        :param original_transaction_id:
        :param x_organization_id: The unique identifier for the organization making the request
        :param external_id: Unique identifier for the credit note in the external system.
        :param date_: Date when the credit note was issued or created.
        :param status: Current state of the credit note in its lifecycle.
        :param currency:
        :param transaction_items: Detailed list of individual items included in this credit note.
        :param description: Brief explanation or reason for issuing the credit note.
        :param total_amount: Total monetary value of the credit note, including all items and taxes.
        :param marketplace: Indicates whether this credit note is associated with a marketplace transaction.
        :param tax_amount_imported: Pre-calculated total tax amount for the entire credit note, if provided by the external system.
        :param tax_rate_imported: Pre-calculated overall tax rate for the credit note, if provided by the external system.
        :param taxable_amount: Total portion of the credit note amount subject to taxation.
        :param addresses: A list of TransactionAddressBuilder objects or None if no addresses are provided. This field represents the addresses associated with the transaction.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesPostRequest(
            original_transaction_id=original_transaction_id,
            x_organization_id=x_organization_id,
            credit_note_create=models.CreditNoteCreate(
                external_id=external_id,
                date_=date_,
                status=status,
                description=description,
                total_amount=total_amount,
                marketplace=marketplace,
                tax_amount_imported=tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                taxable_amount=taxable_amount,
                currency=currency,
                addresses=utils.get_pydantic_model(
                    addresses, OptionalNullable[List[models.TransactionAddressBuilder]]
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.CreditNoteItemCreateUpdate]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/transactions/{original_transaction_id}/credit_notes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.CreateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesPostSecurity,
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.credit_note_create,
                False,
                False,
                "json",
                models.CreditNoteCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_credit_note_by_transaction_id_v1_transactions__original_transaction_id__credit_notes_post",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.TransactionRead, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update_credit_note(
        self,
        *,
        security: Union[
            models.UpdateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesCreditNoteIDPutSecurity,
            models.UpdateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesCreditNoteIDPutSecurityTypedDict,
        ],
        original_transaction_id: str,
        credit_note_id: str,
        x_organization_id: Nullable[str],
        external_id: str,
        date_: datetime,
        status: models.Status,
        currency: models.CurrencyEnum,
        transaction_items: Union[
            List[models.CreditNoteItemCreateUpdate],
            List[models.CreditNoteItemCreateUpdateTypedDict],
        ],
        description: OptionalNullable[str] = UNSET,
        total_amount: Optional[
            Union[
                models.CreditNoteCreateTotalAmount,
                models.CreditNoteCreateTotalAmountTypedDict,
            ]
        ] = None,
        marketplace: OptionalNullable[bool] = UNSET,
        tax_amount_imported: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxAmountImported,
                models.CreditNoteCreateTaxAmountImportedTypedDict,
            ]
        ] = UNSET,
        tax_rate_imported: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxRateImported,
                models.CreditNoteCreateTaxRateImportedTypedDict,
            ]
        ] = UNSET,
        taxable_amount: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxableAmount,
                models.CreditNoteCreateTaxableAmountTypedDict,
            ]
        ] = UNSET,
        addresses: OptionalNullable[
            Union[
                List[models.TransactionAddressBuilder],
                List[models.TransactionAddressBuilderTypedDict],
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Update Credit Note By Transaction Id

        Update an existing credit note for a specific transaction.

        :param security:
        :param original_transaction_id:
        :param credit_note_id:
        :param x_organization_id: The unique identifier for the organization making the request
        :param external_id: Unique identifier for the credit note in the external system.
        :param date_: Date when the credit note was issued or created.
        :param status: Current state of the credit note in its lifecycle.
        :param currency:
        :param transaction_items: Detailed list of individual items included in this credit note.
        :param description: Brief explanation or reason for issuing the credit note.
        :param total_amount: Total monetary value of the credit note, including all items and taxes.
        :param marketplace: Indicates whether this credit note is associated with a marketplace transaction.
        :param tax_amount_imported: Pre-calculated total tax amount for the entire credit note, if provided by the external system.
        :param tax_rate_imported: Pre-calculated overall tax rate for the credit note, if provided by the external system.
        :param taxable_amount: Total portion of the credit note amount subject to taxation.
        :param addresses: A list of TransactionAddressBuilder objects or None if no addresses are provided. This field represents the addresses associated with the transaction.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesCreditNoteIDPutRequest(
            original_transaction_id=original_transaction_id,
            credit_note_id=credit_note_id,
            x_organization_id=x_organization_id,
            credit_note_create=models.CreditNoteCreate(
                external_id=external_id,
                date_=date_,
                status=status,
                description=description,
                total_amount=total_amount,
                marketplace=marketplace,
                tax_amount_imported=tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                taxable_amount=taxable_amount,
                currency=currency,
                addresses=utils.get_pydantic_model(
                    addresses, OptionalNullable[List[models.TransactionAddressBuilder]]
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.CreditNoteItemCreateUpdate]
                ),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/transactions/{original_transaction_id}/credit_notes/{credit_note_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.UpdateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesCreditNoteIDPutSecurity,
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.credit_note_create,
                False,
                False,
                "json",
                models.CreditNoteCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_credit_note_by_transaction_id_v1_transactions__original_transaction_id__credit_notes__credit_note_id__put",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Any, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_credit_note_async(
        self,
        *,
        security: Union[
            models.UpdateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesCreditNoteIDPutSecurity,
            models.UpdateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesCreditNoteIDPutSecurityTypedDict,
        ],
        original_transaction_id: str,
        credit_note_id: str,
        x_organization_id: Nullable[str],
        external_id: str,
        date_: datetime,
        status: models.Status,
        currency: models.CurrencyEnum,
        transaction_items: Union[
            List[models.CreditNoteItemCreateUpdate],
            List[models.CreditNoteItemCreateUpdateTypedDict],
        ],
        description: OptionalNullable[str] = UNSET,
        total_amount: Optional[
            Union[
                models.CreditNoteCreateTotalAmount,
                models.CreditNoteCreateTotalAmountTypedDict,
            ]
        ] = None,
        marketplace: OptionalNullable[bool] = UNSET,
        tax_amount_imported: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxAmountImported,
                models.CreditNoteCreateTaxAmountImportedTypedDict,
            ]
        ] = UNSET,
        tax_rate_imported: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxRateImported,
                models.CreditNoteCreateTaxRateImportedTypedDict,
            ]
        ] = UNSET,
        taxable_amount: OptionalNullable[
            Union[
                models.CreditNoteCreateTaxableAmount,
                models.CreditNoteCreateTaxableAmountTypedDict,
            ]
        ] = UNSET,
        addresses: OptionalNullable[
            Union[
                List[models.TransactionAddressBuilder],
                List[models.TransactionAddressBuilderTypedDict],
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Update Credit Note By Transaction Id

        Update an existing credit note for a specific transaction.

        :param security:
        :param original_transaction_id:
        :param credit_note_id:
        :param x_organization_id: The unique identifier for the organization making the request
        :param external_id: Unique identifier for the credit note in the external system.
        :param date_: Date when the credit note was issued or created.
        :param status: Current state of the credit note in its lifecycle.
        :param currency:
        :param transaction_items: Detailed list of individual items included in this credit note.
        :param description: Brief explanation or reason for issuing the credit note.
        :param total_amount: Total monetary value of the credit note, including all items and taxes.
        :param marketplace: Indicates whether this credit note is associated with a marketplace transaction.
        :param tax_amount_imported: Pre-calculated total tax amount for the entire credit note, if provided by the external system.
        :param tax_rate_imported: Pre-calculated overall tax rate for the credit note, if provided by the external system.
        :param taxable_amount: Total portion of the credit note amount subject to taxation.
        :param addresses: A list of TransactionAddressBuilder objects or None if no addresses are provided. This field represents the addresses associated with the transaction.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesCreditNoteIDPutRequest(
            original_transaction_id=original_transaction_id,
            credit_note_id=credit_note_id,
            x_organization_id=x_organization_id,
            credit_note_create=models.CreditNoteCreate(
                external_id=external_id,
                date_=date_,
                status=status,
                description=description,
                total_amount=total_amount,
                marketplace=marketplace,
                tax_amount_imported=tax_amount_imported,
                tax_rate_imported=tax_rate_imported,
                taxable_amount=taxable_amount,
                currency=currency,
                addresses=utils.get_pydantic_model(
                    addresses, OptionalNullable[List[models.TransactionAddressBuilder]]
                ),
                transaction_items=utils.get_pydantic_model(
                    transaction_items, List[models.CreditNoteItemCreateUpdate]
                ),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/transactions/{original_transaction_id}/credit_notes/{credit_note_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security,
                models.UpdateCreditNoteByTransactionIDV1TransactionsOriginalTransactionIDCreditNotesCreditNoteIDPutSecurity,
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request.credit_note_create,
                False,
                False,
                "json",
                models.CreditNoteCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_credit_note_by_transaction_id_v1_transactions__original_transaction_id__credit_notes__credit_note_id__put",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Any, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)
